@@ -244,7 +244,7 @@ void decompose(const CGAL_Nef_polyhedron3 *N, Output out_iter)
   assert(N);
   CGAL_Polyhedron poly;
   if (N->is_simple()) {
-    nefworkaround::convert_to_Polyhedron<CGAL_Kernel3>(*N, poly);
+    N->convert_to_polyhedron(poly);
   }
   if (is_weakly_convex(poly)) {
     PRINTD("Minkowski: Object is convex and Nef");
@@ -301,7 +301,7 @@ Geometry const * minkowskitest(const Geometry::Geometries &children)
       std::vector<PolyhedronK> convexP[2]; // Stores decomposed operands 
       std::list<PolyhedronK> result_parts;
 
-      for (int i = 0; i < 2; i++) {
+      for (int i = 0; i < 2; ++i) {
         shared_ptr<const CGAL_Nef_polyhedron> N;
         if (const PolySet *ps = dynamic_cast<const PolySet *>(operands[i])) {
           if (ps->is_convex()) {
@@ -318,7 +318,7 @@ Geometry const * minkowskitest(const Geometry::Geometries &children)
         else if (const CGAL_Nef_polyhedron *n = dynamic_cast<const CGAL_Nef_polyhedron *>(operands[i])) {
           CGAL_Polyhedron poly;
           if (n->p3->is_simple()) {
-            nefworkaround::convert_to_Polyhedron<CGAL_Kernel3>(*n->p3, poly);
+            n->p3->convert_to_polyhedron(poly);
             // FIXME: Can we calculate weakly_convex on a PolyhedronK instead?
             if (is_weakly_convex(poly)) {
               PRINTDB("Minkowski: child %d is convex and Nef", i);
@@ -345,15 +345,15 @@ Geometry const * minkowskitest(const Geometry::Geometries &children)
         std::vector<K::Point_3> minkowski_points;
         
         // For each permutation of convex operands..
-        BOOST_FOREACH(const PolyhedronK &p0, convexP[0]) {
-          BOOST_FOREACH(const PolyhedronK &p1, convexP[1]) {
+        for(const PolyhedronK &p0 : convexP[0]) {
+          for(const PolyhedronK &p1 : convexP[1]) {
             t.start();
             
             // Create minkowski pointcloud
             minkowski_points.clear();
             minkowski_points.reserve(p0.size_of_vertices() * p0.size_of_vertices());
-            BOOST_FOREACH(const K::Point_3 &p0p, std::make_pair(p0.points_begin(), p0.points_end())) {
-              BOOST_FOREACH(const K::Point_3 &p1p, std::make_pair(p1.points_begin(), p1.points_end())) {
+            for(const K::Point_3 &p0p : std::make_pair(p0.points_begin(), p0.points_end())) {
+              for(const K::Point_3 &p1p : std::make_pair(p1.points_begin(), p1.points_end())) {
                 minkowski_points.push_back(p0p+(p1p-CGAL::ORIGIN));
               }
             }
@@ -420,7 +420,7 @@ Geometry const * minkowskitest(const Geometry::Geometries &children)
         }
       }
       
-      if (minkowski_ch_it != boost::next(children.begin())) delete operands[0];
+      if (minkowski_ch_it != std::next(children.begin())) delete operands[0];
       
       if (result_parts.size() == 1) {
         PolySet *ps = new PolySet(3,true);
@@ -436,7 +436,7 @@ Geometry const * minkowskitest(const Geometry::Geometries &children)
           fake_children.push_back(std::make_pair((const AbstractNode*)NULL,
                                                  shared_ptr<const Geometry>(createNefPolyhedronFromGeometry(ps))));
         }
-        CGAL_Nef_polyhedron *N = CGALUtils::applyOperator(fake_children, OPENSCAD_UNION);
+        CGAL_Nef_polyhedron *N = CGALUtils::applyUnion3D(fake_children.begin(), fake_children.end());
         t.stop();
         if (N) PRINTDB("Minkowski: Union done: %f s",t.time());
         else PRINTDB("Minkowski: Union failed: %f s",t.time());
@@ -456,7 +456,7 @@ Geometry const * minkowskitest(const Geometry::Geometries &children)
     // If anything throws we simply fall back to Nef Minkowski
     PRINTD("Minkowski: Falling back to Nef Minkowski");
     
-    CGAL_Nef_polyhedron *N = applyOperator(children, OPENSCAD_MINKOWSKI);
+    CGAL_Nef_polyhedron *N = applyOperator3D(children, OPENSCAD_MINKOWSKI);
     return N;
   }
 }
@@ -497,7 +497,7 @@ void read_stl_facet( std::ifstream &f, stl_facet &facet )
 {
 	f.read( (char*)facet.data8, STL_FACET_NUMBYTES );
 #ifdef BOOST_BIG_ENDIAN
-	for ( int i = 0; i < 12; i++ ) {
+	for ( int i = 0; i < 12; ++i ) {
 		uint32_byte_swap( facet.data32[ i ] );
 	}
 	// we ignore attribute byte count
@@ -511,7 +511,7 @@ PolySet *import_stl(const std::string &filename)
   // Open file and position at the end
   std::ifstream f(filename.c_str(), std::ios::in | std::ios::binary | std::ios::ate);
   if (!f.good()) {
-    PRINTB("WARNING: Can't open import file '%s'.", filename);
+    LOG(message_group::Warning,Location::None,"","Can't open import file: %1$s",filename);
     return NULL;
   }
 
@@ -555,12 +555,12 @@ PolySet *import_stl(const std::string &filename)
       boost::smatch results;
       if (boost::regex_search(line, results, ex_vertices)) {
         try {
-          for (int v=0;v<3;v++) {
+          for (int v=0; v<3; ++v) {
             vdata[i][v] = boost::lexical_cast<double>(results[v+1]);
           }
         }
         catch (const boost::bad_lexical_cast &blc) {
-          PRINTB("WARNING: Can't parse vertex line '%s'.", line);
+          LOG(message_group::Warning,Location::None,"","Can't parse vertex line: %1$s",line);
           i = 10;
           continue;
         }
@@ -650,7 +650,7 @@ int main(int argc, char *argv[])
         std::cerr << "Error importing STL " << filename << std::endl;
         exit(1);
       }
-      std::cerr << "Imported " << ps->numPolygons() << " polygons" << std::endl;
+      std::cerr << "Imported " << ps->numFacets() << " polygons" << std::endl;
     }
     else if (suffix == ".nef3") {
       N = new CGAL_Nef_polyhedron(new CGAL_Nef_polyhedron3);
@@ -672,7 +672,7 @@ int main(int argc, char *argv[])
   std::cerr << "Decomposed into " << result.size() << " convex parts" << std::endl;
 
   int idx = 0;
-  BOOST_FOREACH(const PolyhedronK &P, result) {
+  for(const PolyhedronK &P : result) {
     PolySet *result_ps = new PolySet(3);
     if (CGALUtils::createPolySetFromPolyhedron(P, *result_ps)) {
       std::cerr << "Error converting to PolySet\n";
