@@ -17,7 +17,7 @@
 #pragma pop_macro("NDEBUG")
 
 using namespace CGALUtils;
-namespace fs=boost::fileystem;
+namespace fs=boost::filesystem;
 
 #define STL_FACET_NUMBYTES 4*3*4+2
 // as there is no 'float32_t' standard, we assume the systems 'float'
@@ -55,7 +55,7 @@ void read_stl_facet( std::ifstream &f, stl_facet &facet )
 {
 	f.read( (char*)facet.data8, STL_FACET_NUMBYTES );
 #ifdef BOOST_BIG_ENDIAN
-	for ( int i = 0; i < 12; i++ ) {
+	for ( int i = 0; i < 12; ++i ) {
 		uint32_byte_swap( facet.data32[ i ] );
 	}
 	// we ignore attribute byte count
@@ -69,7 +69,7 @@ PolySet *import_stl(const std::string &filename)
   // Open file and position at the end
   std::ifstream f(filename.c_str(), std::ios::in | std::ios::binary | std::ios::ate);
   if (!f.good()) {
-    PRINTB("WARNING: Can't open import file '%s'.", filename);
+    LOG(message_group::Warning,Location::None,"","Can't open import file: %1$s",filename);
     return NULL;
   }
 
@@ -113,12 +113,12 @@ PolySet *import_stl(const std::string &filename)
       boost::smatch results;
       if (boost::regex_search(line, results, ex_vertices)) {
         try {
-          for (int v=0;v<3;v++) {
+          for (int v=0; v<3; ++v) {
             vdata[i][v] = boost::lexical_cast<double>(results[v+1]);
           }
         }
         catch (const boost::bad_lexical_cast &blc) {
-          PRINTB("WARNING: Can't parse vertex line '%s'.", line);
+          LOG(message_group::Warning,Location::None,"","Can't parse vertex line: %1$s",line);
           i = 10;
           continue;
         }
@@ -163,7 +163,7 @@ int main(int argc, char *argv[])
         std::cerr << "Error importing STL " << argv[1] << std::endl;
         exit(1);
       }
-      std::cerr << "Imported " << ps->numPolygons() << " polygons" << std::endl;
+      std::cerr << "Imported " << ps->numFacets() << " polygons" << std::endl;
     }
     else if (suffix == ".nef3") {
       N = new CGAL_Nef_polyhedron(new CGAL_Nef_polyhedron3);
