@@ -14,6 +14,8 @@
 # - MacPorts: curl, cmake
 #
 
+set -e
+
 if [ "`echo $* | grep \\\-v `" ]; then
   set -x
 fi
@@ -27,12 +29,12 @@ OPTION_DEPLOY=false
 OPTION_FORCE=0
 
 PACKAGES=(
-    "double_conversion 3.1.1"
+    "double_conversion 3.1.5"
     "eigen 3.3.7"
     "gmp 6.1.2"
     "mpfr 4.0.2"
     "glew 2.1.0"
-    "gettext 0.19.8"
+    "gettext 0.21"
     "libffi 3.2.1"
     "freetype 2.9.1"
     "ragel 6.10"
@@ -40,15 +42,18 @@ PACKAGES=(
     "libzip 1.5.1"
     "libxml2 2.9.9"
     "fontconfig 2.13.1"
-    "hidapi 0.8.0-rc1"
+    "hidapi 0.9.0"
     "libuuid 1.6.2"
     "lib3mf 1.8.1"
     "glib2 2.56.3"
-    "boost 1.65.1"
-    "cgal 4.13"
-    "qt5 5.9.7"
+    "boost 1.74.0"
+    "poppler 21.01.0"
+    "pixman 0.40.0"
+    "cairo 1.16.0"
+    "cgal 4.14.3"
+    "qt5 5.9.9"
     "opencsg 1.4.2"
-    "qscintilla 2.10.8"
+    "qscintilla 2.11.6"
 )
 DEPLOY_PACKAGES=(
     "sparkle 1.21.3"
@@ -88,6 +93,33 @@ package_version()
     return 1
 }
 
+# Usage: check_version_file <package> <version>
+# Checks if $DEPLOYDIR/fileshare/macosx-build-dependencies/$package.version exists
+# and its contents equals $version
+# Returns success (0) if it does
+check_version_file()
+{
+    versionfile="$DEPLOYDIR/share/macosx-build-dependencies/$1.version"
+    if [ -f $versionfile ]; then
+	[[ $(cat $versionfile) == $2 ]]
+	return $?
+    else
+	return 1
+    fi
+}
+
+# Usage: is_installed <package> [<version>]
+# Returns success (0) if the/a version of the package is already installed
+is_installed()
+{
+    if check_version_file $1 $2; then
+	echo "$1 $2 already installed - not building"
+	return 0
+    else
+	return 1
+    fi
+}
+
 # Usage: build <package> <version>
 build()
 {
@@ -96,8 +128,9 @@ build()
 
     local should_install=$(( $OPTION_FORCE == 1 ))
     if [[ $should_install == 0 ]]; then
-        is_installed $package $version
-        should_install=$?
+        if ! is_installed $package $version; then
+            should_install=1
+	fi
     fi
     if [[ $should_install == 1 ]]; then
         set -e
@@ -106,39 +139,6 @@ build()
     fi
 }
 
-# Usage: is_installed <package> [<version>]
-# Returns success (0) if the/a version of the package is already installed
-is_installed()
-{
-    if check_$1 $2; then
-      echo "$1 already installed - not building"
-      return 0
-    fi
-    return 1
-}
-
-# Usage: check_dir <dir>
-# Checks if $DEPLOYDIR/<dir> exists and is a folder
-# Returns success (0) if the folder exists
-check_dir()
-{
-    test -d "$DEPLOYDIR/$1"
-}
-
-# Usage: check_file <file>
-# Checks if $DEPLOYDIR/<file> exists and is a file
-# Returns success (0) if the file exists
-check_file()
-{
-    test -f "$DEPLOYDIR/$1"
-}
-
-
-check_double_conversion()
-{
-    check_file lib/libdouble-conversion.a
-}
-
 build_double_conversion()
 {
   version="$1"
@@ -154,11 +154,7 @@ build_double_conversion()
   cmake -DCMAKE_INSTALL_PREFIX=$DEPLOYDIR -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" .
   make -j$NUMCPU
   make install
-}
-
-check_qt5()
-{
-    check_dir lib/QtCore.framework
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/double_conversion.version
 }
 
 build_qt5()
@@ -168,17 +164,17 @@ build_qt5()
   echo "Building Qt" $version "..."
   cd $BASEDIR/src
   v=(${version//./ }) # Split into array
-  rm -rf qt-opensource-opensource-src-$version
+  rm -rf qt-everywhere-opensource-src-$version
   if [ ! -f qt-everywhere-opensource-src-$version.tar.xz ]; then
       curl -LO http://download.qt.io/official_releases/qt/${v[0]}.${v[1]}/$version/single/qt-everywhere-opensource-src-$version.tar.xz
   fi
   tar xzf qt-everywhere-opensource-src-$version.tar.xz
   cd qt-everywhere-opensource-src-$version
-  patch -p1 < $OPENSCADDIR/patches/qt5/qt-5.9.7-macos.patch 
+  patch -p1 < $OPENSCADDIR/patches/qt5/qt-5.9.7-macos.patch
   ./configure -prefix $DEPLOYDIR -release -opensource -confirm-license \
 		-nomake examples -nomake tests \
 		-no-xcb -no-glib -no-harfbuzz -no-sql-db2 -no-sql-ibase -no-sql-mysql -no-sql-oci -no-sql-odbc \
-		-no-sql-psql -no-sql-sqlite2 -no-sql-tds -no-cups \
+		-no-sql-psql -no-sql-sqlite -no-sql-sqlite2 -no-sql-tds -no-cups -no-assimp -no-qml-debug \
                 -skip qtx11extras -skip qtandroidextras -skip qtserialport -skip qtserialbus \
                 -skip qtactiveqt -skip qtxmlpatterns -skip qtdeclarative -skip qtscxml \
                 -skip qtpurchasing -skip qtcanvas3d -skip qtwayland \
@@ -186,14 +182,11 @@ build_qt5()
                 -skip qtdatavis3d -skip qtcharts -skip qtwinextras \
                 -skip qtgraphicaleffects -skip qtquickcontrols2 -skip qtquickcontrols \
                 -skip qtvirtualkeyboard -skip qtlocation -skip qtwebengine -skip qtwebview \
-                -skip qtscript -skip qttranslations -skip qtdoc
+                -skip qtscript -skip qttranslations -skip qtdoc \
+                -no-feature-openal -no-feature-avfoundation
   make -j"$NUMCPU" 
   make install
-}
-
-check_qscintilla()
-{
-    check_file include/Qsci/qsciscintilla.h 
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/qt5.version
 }
 
 build_qscintilla()
@@ -201,21 +194,18 @@ build_qscintilla()
   version=$1
   echo "Building QScintilla" $version "..."
   cd $BASEDIR/src
-  rm -rf QScintilla_gpl-$version
-  if [ ! -f QScintilla_gpl-$version.tar.gz ]; then
-    curl -LO http://downloads.sourceforge.net/project/pyqt/QScintilla2/QScintilla-$version/QScintilla_gpl-$version.tar.gz
+  QSCINTILLA_FILENAME="QScintilla-$version.tar.gz"
+  rm -rf "${QSCINTILLA_FILENAME}"
+  if [ ! -f "${QSCINTILLA_FILENAME}" ]; then
+      curl -LO https://www.riverbankcomputing.com/static/Downloads/QScintilla/$version/"${QSCINTILLA_FILENAME}"
   fi
-  tar xzf QScintilla_gpl-$version.tar.gz
-  cd QScintilla_gpl-$version/Qt4Qt5
-#  patch -p2 < $OPENSCADDIR/patches/QScintilla-2.9.3-xcode8.patch
+  tar xzf "${QSCINTILLA_FILENAME}"
+  cd QScintilla*/Qt4Qt5
+  #patch -p2 < $OPENSCADDIR/patches/QScintilla-2.9.3-xcode8.patch
   qmake qscintilla.pro
   make -j"$NUMCPU" install
   install_name_tool -id @rpath/libqscintilla2_qt5.dylib $DEPLOYDIR/lib/libqscintilla2_qt5.dylib
-}
-
-check_gmp()
-{
-    check_file lib/libgmp.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/qscintilla.version
 }
 
 build_gmp()
@@ -237,13 +227,7 @@ build_gmp()
   install_name_tool -id @rpath/libgmp.dylib $DEPLOYDIR/lib/libgmp.dylib
   install_name_tool -id @rpath/libgmpxx.dylib $DEPLOYDIR/lib/libgmpxx.dylib
   install_name_tool -change $DEPLOYDIR/lib/libgmp.10.dylib @rpath/libgmp.dylib $DEPLOYDIR/lib/libgmpxx.dylib
-}
-
-# As with gmplib, mpfr is built separately in 32-bit and 64-bit mode and then merged
-# afterwards.
-check_mpfr()
-{
-    check_file include/mpfr.h
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/gmp.version
 }
 
 build_mpfr()
@@ -263,11 +247,7 @@ build_mpfr()
   make -j"$NUMCPU" install
 
   install_name_tool -id @rpath/libmpfr.dylib $DEPLOYDIR/lib/libmpfr.dylib
-}
-
-check_boost()
-{
-    check_file lib/libboost_system.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/mpfr.version
 }
 
 build_boost()
@@ -287,11 +267,7 @@ build_boost()
   BOOST_TOOLSET="toolset=clang"
   echo "using clang ;" >> tools/build/user-config.jam 
   ./b2 -j"$NUMCPU" -d+2 $BOOST_TOOLSET cflags="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64" linkflags="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64 -headerpad_max_install_names" install
-}
-
-check_cgal()
-{
-    check_file lib/libCGAL.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/boost.version
 }
 
 build_cgal()
@@ -312,11 +288,7 @@ build_cgal()
   install_name_tool -id @rpath/libCGAL.dylib $DEPLOYDIR/lib/libCGAL.dylib
   install_name_tool -id @rpath/libCGAL_Core.dylib $DEPLOYDIR/lib/libCGAL_Core.dylib
   install_name_tool -change libCGAL.11.dylib @rpath/libCGAL.dylib $DEPLOYDIR/lib/libCGAL_Core.dylib
-}
-
-check_glew()
-{
-    check_file lib/libGLEW.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/cgal.version
 }
 
 build_glew()
@@ -333,11 +305,7 @@ build_glew()
   cd glew-$version
   mkdir -p $DEPLOYDIR/lib/pkgconfig
   make GLEW_DEST=$DEPLOYDIR CFLAGS.EXTRA="-no-cpp-precomp -dynamic -fno-common -mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64" LDFLAGS.EXTRA="-install_name @rpath/libGLEW.dylib -mmacosx-version-min=$MAC_OSX_VERSION_MIN -arch x86_64" POPT="-Os" STRIP= install
-}
-
-check_opencsg()
-{
-    check_file lib/libopencsg.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/glew.version
 }
 
 build_opencsg()
@@ -356,21 +324,9 @@ build_opencsg()
   qmake -r INSTALLDIR=$DEPLOYDIR
   make install
   install_name_tool -id @rpath/libopencsg.dylib $DEPLOYDIR/lib/libopencsg.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/opencsg.version
 }
 
-# Usage: func [<version>]
-check_eigen()
-{
-    # To check version:
-    # include/eigen3/Eigen/src/Core/util/Macros.h:
-    #  #define EIGEN_WORLD_VERSION 3
-    #  #define EIGEN_MAJOR_VERSION 2
-    #  #define EIGEN_MINOR_VERSION 3
-
-    check_dir include/eigen3
-}
-
-# Usage: func <version>
 build_eigen()
 {
   version=$1
@@ -380,23 +336,18 @@ build_eigen()
   rm -rf eigen-$version
 
   if [ ! -f eigen-$version.tar.bz2 ]; then
-    curl -LO http://bitbucket.org/eigen/eigen/get/$version.tar.bz2
-    mv $version.tar.bz2 eigen-$version.tar.bz2
+    curl -LO https://gitlab.com/libeigen/eigen/-/archive/$version/eigen-$version.tar.bz2
   fi
   EIGENDIR=`tar tjf eigen-$version.tar.bz2 | head -1 | cut -f1 -d"/"`
-  rm -rf $EIGENDIR
+  rm -rf "./$EIGENDIR"
   tar xjf eigen-$version.tar.bz2
-  ln -s ./$EIGENDIR eigen-$version
+  ln -s "./$EIGENDIR" eigen-$version || true
   cd eigen-$version
   mkdir build
   cd build
   cmake -DCMAKE_INSTALL_PREFIX=$DEPLOYDIR -DEIGEN_TEST_NOQT=TRUE -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" -DCMAKE_OSX_ARCHITECTURES="x86_64" ..
   make -j"$NUMCPU" install
-}
-
-check_sparkle()
-{
-    check_file lib/Sparkle.framework/Sparkle
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/eigen.version
 }
 
 # Usage:
@@ -441,11 +392,8 @@ build_sparkle()
 #  rm -rf $DEPLOYDIR/lib/Sparkle.framework
 #  cp -Rf build/Release/Sparkle.framework $DEPLOYDIR/lib/ 
 #  Install_name_tool -id $DEPLOYDIR/lib/Sparkle.framework/Versions/A/Sparkle $DEPLOYDIR/lib/Sparkle.framework/Sparkle
-}
 
-check_freetype()
-{
-    check_file lib/libfreetype.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/sparkle.version
 }
 
 build_freetype()
@@ -468,13 +416,9 @@ build_freetype()
   make -j"$NUMCPU"
   make install
   install_name_tool -id @rpath/libfreetype.dylib $DEPLOYDIR/lib/libfreetype.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/freetype.version
 }
  
-check_libzip()
-{
-    check_file lib/libzip.dylib
-}
-
 build_libzip()
 {
   version="$1"
@@ -491,11 +435,7 @@ build_libzip()
   make -j$NUMCPU
   make install
   install_name_tool -id @rpath/libzip.dylib $DEPLOYDIR/lib/libzip.dylib
-}
-
-check_libxml2()
-{
-    check_file lib/libxml2.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/libzip.version
 }
 
 build_libxml2()
@@ -514,11 +454,7 @@ build_libxml2()
   make -j$NUMCPU
   make install
   install_name_tool -id @rpath/libxml2.dylib $DEPLOYDIR/lib/libxml2.dylib
-}
-
-check_fontconfig()
-{
-    check_file lib/libfontconfig.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/libxml2.version
 }
 
 build_fontconfig()
@@ -533,19 +469,13 @@ build_fontconfig()
   fi
   tar xzf "fontconfig-$version.tar.gz"
   cd "fontconfig-$version"
-  export PKG_CONFIG_PATH="$DEPLOYDIR/lib/pkgconfig"
   # FIXME: The "ac_cv_func_mkostemp=no" is a workaround for fontconfig's autotools config not respecting any passed
   # -no_weak_imports linker flag. This may be improved in future versions of fontconfig
-  ./configure --prefix="$DEPLOYDIR" --enable-libxml2 CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN ac_cv_func_mkostemp=no
-  unset PKG_CONFIG_PATH
+  ./configure --prefix="$DEPLOYDIR" --enable-libxml2 CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS="-Wl,-rpath,$DEPLOYDIR/lib -mmacosx-version-min=$MAC_OSX_VERSION_MIN" ac_cv_func_mkostemp=no
   make -j$NUMCPU
   make install
   install_name_tool -id @rpath/libfontconfig.dylib $DEPLOYDIR/lib/libfontconfig.dylib
-}
-
-check_libffi()
-{
-    check_file lib/libffi.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/fontconfig.version
 }
 
 build_libffi()
@@ -564,11 +494,7 @@ build_libffi()
   make -j$NUMCPU
   make install
   install_name_tool -id @rpath/libffi.dylib $DEPLOYDIR/lib/libffi.dylib
-}
-
-check_gettext()
-{
-    check_file lib/libgettextlib.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/libffi.version
 }
 
 build_gettext()
@@ -583,16 +509,20 @@ build_gettext()
   fi
   tar xzf "gettext-$version.tar.gz"
   cd "gettext-$version"
-  patch -p1 < $OPENSCADDIR/patches/gettext.patch
-  ./configure --with-included-glib --prefix="$DEPLOYDIR" CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN
+  #patch -p1 < $OPENSCADDIR/patches/gettext.patch
+  ./configure --with-included-glib --disable-java --disable-csharp --prefix="$DEPLOYDIR" CFLAGS=-mmacosx-version-min=$MAC_OSX_VERSION_MIN LDFLAGS="-mmacosx-version-min=$MAC_OSX_VERSION_MIN -Wl,-rpath,$DEPLOYDIR/lib"
   make -j$NUMCPU
   make install
-  install_name_tool -id $DEPLOYDIR/lib/libintl.dylib $DEPLOYDIR/lib/libintl.dylib
-}
+  install_name_tool -id @rpath/libintl.dylib $DEPLOYDIR/lib/libintl.dylib
+  install_name_tool -id @rpath/libgettextlib.dylib $DEPLOYDIR/lib/libgettextlib-$version.dylib
 
-check_glib2()
-{
-    check_file lib/libglib-2.0.dylib
+  install_name_tool -change $DEPLOYDIR/lib/libintl.9.dylib @rpath/libintl.dylib $DEPLOYDIR/lib/libgettextlib-$version.dylib
+
+  install_name_tool -change $DEPLOYDIR/lib/libgettextsrc-$version.dylib @rpath/libgettextsrc.dylib $DEPLOYDIR/bin/msgfmt
+  install_name_tool -change $DEPLOYDIR/lib/libgettextlib-$version.dylib @rpath/libgettextlib.dylib $DEPLOYDIR/bin/msgfmt
+  install_name_tool -change $DEPLOYDIR/lib/libintl.9.dylib @rpath/libintl.dylib $DEPLOYDIR/bin/msgfmt
+
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/gettext.version
 }
 
 build_glib2()
@@ -610,17 +540,11 @@ build_glib2()
   tar xJf "glib-$version.tar.xz"
   cd "glib-$version"
 
-  export PKG_CONFIG_PATH="$DEPLOYDIR/lib/pkgconfig"
-  ./configure --disable-gtk-doc --disable-man --without-pcre --prefix="$DEPLOYDIR" CFLAGS="-I$DEPLOYDIR/include -mmacosx-version-min=$MAC_OSX_VERSION_MIN" LDFLAGS="-L$DEPLOYDIR/lib -mmacosx-version-min=$MAC_OSX_VERSION_MIN"
-  unset PKG_CONFIG_PATH
+  ./configure --disable-gtk-doc --disable-man --without-pcre --prefix="$DEPLOYDIR" CFLAGS="-I$DEPLOYDIR/include -mmacosx-version-min=$MAC_OSX_VERSION_MIN" LDFLAGS="-Wl,-rpath,$DEPLOYDIR/lib -L$DEPLOYDIR/lib -mmacosx-version-min=$MAC_OSX_VERSION_MIN"
   make -j$NUMCPU
   make install
   install_name_tool -id @rpath/libglib-2.0.dylib $DEPLOYDIR/lib/libglib-2.0.dylib
-}
-
-check_ragel()
-{
-    check_file bin/ragel
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/glib2.version
 }
 
 build_ragel()
@@ -638,11 +562,7 @@ build_ragel()
   ./configure --prefix="$DEPLOYDIR"
   make -j$NUMCPU
   make install
-}
-
-check_harfbuzz()
-{
-    check_file lib/libharfbuzz.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/ragel.version
 }
 
 build_harfbuzz()
@@ -663,11 +583,7 @@ build_harfbuzz()
   make -j$NUMCPU
   make install
   install_name_tool -id @rpath/libharfbuzz.dylib $DEPLOYDIR/lib/libharfbuzz.dylib
-}
-
-check_hidapi()
-{
-    check_file lib/libhidapi.a
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/harfbuzz.version
 }
 
 build_hidapi()
@@ -678,7 +594,7 @@ build_hidapi()
   cd "$BASEDIR"/src
   rm -rf "hidapi-hidapi-$version"
   if [ ! -f "hidapi-$version.zip" ]; then
-    curl --insecure -LO "https://github.com/signal11/hidapi/archive/hidapi-${version}.zip"
+    curl --insecure -LO "https://github.com/libusb/hidapi/archive/hidapi-${version}.zip"
   fi
   unzip "hidapi-$version.zip"
   cd "hidapi-hidapi-$version"
@@ -686,11 +602,7 @@ build_hidapi()
   ./configure --prefix=$DEPLOYDIR CFLAGS="-mmacosx-version-min=$MAC_OSX_VERSION_MIN" LDFLAGS="-mmacosx-version-min=$MAC_OSX_VERSION_MIN"
   make -j"$NUMCPU" install
   install_name_tool -id @rpath/libhidapi.dylib $DEPLOYDIR/lib/libhidapi.dylib
-}
-
-check_libuuid()
-{
-    check_file lib/libuuid.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/hidapi.version
 }
 
 build_libuuid()
@@ -708,11 +620,7 @@ build_libuuid()
   make -j"$NUMCPU"
   make install
   install_name_tool -id @rpath/libuuid.dylib $DEPLOYDIR/lib/libuuid.dylib
-}
-
-check_lib3mf()
-{
-    check_file lib/lib3mf.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/libuuid.version
 }
 
 build_lib3mf()
@@ -730,6 +638,90 @@ build_lib3mf()
   cmake -DLIB3MF_TESTS=false -DCMAKE_PREFIX_PATH=$DEPLOYDIR -DCMAKE_INSTALL_PREFIX=$DEPLOYDIR  -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" .
   make -j"$NUMCPU" VERBOSE=1
   make -j"$NUMCPU" install
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/lib3mf.version
+}
+
+build_poppler()
+{
+  version=$1
+  POPPLER_DIR="poppler-${version}"
+  POPPLER_FILENAME="${POPPLER_DIR}.tar.xz"
+
+  echo "Building poppler" $version "..."
+
+  cd $BASEDIR/src
+  rm -rf "$POPPLER_DIR"
+  if [ ! -f "${POPPLER_FILENAME}" ]; then
+    curl -LO https://poppler.freedesktop.org/"${POPPLER_FILENAME}"
+  fi
+  tar xzf "${POPPLER_FILENAME}"
+  cd "$POPPLER_DIR"
+  mkdir build
+  cd build
+  cmake .. \
+        -DCMAKE_INSTALL_PREFIX="$DEPLOYDIR" \
+        -DCMAKE_OSX_ARCHITECTURES="x86_64" \
+        -DCMAKE_OSX_DEPLOYMENT_TARGET="$MAC_OSX_VERSION_MIN" \
+        -DBUILD_GTK_TESTS=OFF -DBUILD_QT5_TESTS=OFF -DBUILD_QT6_TESTS=OFF \
+        -DBUILD_CPP_TESTS=OFF -DENABLE_GTK_DOC=OFF -DENABLE_QT5=OFF \
+        -DENABLE_QT6=OFF -DENABLE_LIBOPENJPEG=none -DENABLE_DCTDECODER=none \
+        -DENABLE_UTILS=OFF
+  make -j"$NUMCPU" install
+  otool -L $DEPLOYDIR/lib/"libpoppler.dylib"
+  install_name_tool -id @rpath/libpoppler.dylib $DEPLOYDIR/lib/"libpoppler.dylib"
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/poppler.version
+}
+
+build_pixman()
+{
+  version=$1
+  PIXMAN_DIR="pixman-${version}"
+  PIXMAN_FILENAME="${PIXMAN_DIR}.tar.gz"
+
+  echo "Building pixman" $version "..."
+
+  cd $BASEDIR/src
+  rm -rf "$PIXMAN_DIR"
+  if [ ! -f "${PIXMAN_FILENAME}" ]; then
+    curl -LO https://www.cairographics.org/releases/"${PIXMAN_FILENAME}"
+  fi
+  tar xzf "${PIXMAN_FILENAME}"
+  cd "$PIXMAN_DIR"
+  ./configure --prefix=$DEPLOYDIR CXXFLAGS="$CXXSTDFLAGS" CFLAGS="-mmacosx-version-min=$MAC_OSX_VERSION_MIN" LDFLAGS="$LDSTDFLAGS -mmacosx-version-min=$MAC_OSX_VERSION_MIN"
+  make -j"$NUMCPU" install
+  otool -L $DEPLOYDIR/lib/"libpixman-1.dylib"
+  install_name_tool -id @rpath/libpixman-1.dylib $DEPLOYDIR/lib/"libpixman-1.dylib"
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/pixman.version
+}
+
+build_cairo()
+{
+  version=$1
+  CAIRO_DIR="cairo-${version}"
+  CAIRO_FILENAME="${CAIRO_DIR}.tar.xz"
+
+  echo "Building cairo" $version "..."
+
+  cd $BASEDIR/src
+  rm -rf "$CAIRO_DIR"
+  if [ ! -f "${CAIRO_FILENAME}" ]; then
+    curl -LO https://www.cairographics.org/releases/"${CAIRO_FILENAME}"
+  fi
+  tar xzf "${CAIRO_FILENAME}"
+  cd "$CAIRO_DIR"
+  ./configure --prefix=$DEPLOYDIR \
+        CXXFLAGS="$CXXSTDFLAGS" \
+        CFLAGS="-mmacosx-version-min=$MAC_OSX_VERSION_MIN" \
+        LDFLAGS="$LDSTDFLAGS -mmacosx-version-min=$MAC_OSX_VERSION_MIN" \
+        --enable-xlib=no --enable-xlib-xrender=no --enable-xcb=no \
+        --enable-xlib-xcb=no --enable-xcb-shm=no --enable-win32=no \
+        --enable-win32-font=no --enable-png=no --enable-ps=no \
+        --enable-svg=no
+  make -j"$NUMCPU" install
+  otool -L $DEPLOYDIR/lib/libcairo.dylib
+  install_name_tool -id @rpath/libcairo.dylib $DEPLOYDIR/lib/libcairo.dylib
+  install_name_tool -change @rpath/libpixman.dylib @rpath/libpixman-1.dylib $DEPLOYDIR/lib/libcairo.dylib
+  echo $version > $DEPLOYDIR/share/macosx-build-dependencies/cairo.version
 }
 
 if [ ! -f $OPENSCADDIR/openscad.pro ]; then
@@ -785,7 +777,7 @@ if (( $OPTION_FORCE )); then
 fi
 
 echo "Using basedir:" $BASEDIR
-mkdir -p $SRCDIR $DEPLOYDIR
+mkdir -p $SRCDIR $DEPLOYDIR $DEPLOYDIR/share/macosx-build-dependencies
 
 # Only build deploy packages in deploy mode
 if $OPTION_DEPLOY; then
