@@ -1,6 +1,6 @@
 #pragma once
 
-#include "openscad.h"
+#include "version.h"
 #include "qtgettext.h"
 #include "ui_AboutDialog.h"
 
@@ -11,8 +11,6 @@ public:
 	AboutDialog(QWidget *) {
 		setupUi(this);
 		this->setWindowTitle( QString(_("About OpenSCAD")) + " " + openscad_shortversionnumber.c_str());
-		QUrl flattr_qurl(":icons/flattr.png" );
-		this->aboutText->loadResource( QTextDocument::ImageResource, flattr_qurl );
 		QString tmp = this->aboutText->toHtml();
 		tmp.replace("__VERSION__", openscad_detailedversionnumber.c_str());
 		this->aboutText->setHtml(tmp);
