@@ -1,5 +1,6 @@
-#include <AppleEvents.h>
+#include "AppleEvents.h"
 #include <MacTypes.h>
+#include <AssertMacros.h>
 #include <CoreServices/CoreServices.h>
 #include <QApplication>
 #include "MainWindow.h"
@@ -11,8 +12,8 @@ extern "C" {
 OSErr eventHandler(const AppleEvent *, AppleEvent *, SRefCon )
 {
 // FIXME: Ugly hack; just using the first MainWindow we can find
-	MainWindow *mainwin = NULL;
-	foreach (QWidget *w, QApplication::topLevelWidgets()) {
+	MainWindow *mainwin = nullptr;
+	for (auto &w : QApplication::topLevelWidgets()) {
 		mainwin = qobject_cast<MainWindow*>(w);
 		if (mainwin) break;
 	}
@@ -25,7 +26,7 @@ OSErr eventHandler(const AppleEvent *, AppleEvent *, SRefCon )
 void installAppleEventHandlers()
 {
 	// Reload handler
-  OSErr err = AEInstallEventHandler('SCAD', 'relo', NewAEEventHandlerUPP(eventHandler), 0, true);
+  auto err = AEInstallEventHandler('SCAD', 'relo', NewAEEventHandlerUPP(eventHandler), nullptr, true);
   __Require_noErr(err, CantInstallAppleEventHandler);
 	return;
 
