@@ -15,8 +15,8 @@ public:
 	bool contains(const std::string &id) const { return this->cache.contains(id); }
 	shared_ptr<const class CGAL_Nef_polyhedron> get(const std::string &id) const;
 	bool insert(const std::string &id, const shared_ptr<const CGAL_Nef_polyhedron> &N);
-	size_t maxSize() const;
-	void setMaxSize(size_t limit);
+	size_t maxSizeMB() const;
+	void setMaxSizeMB(size_t limit);
 	void clear();
 	void print();
 
