@@ -17,28 +17,33 @@ projection, Perspective and Orthogonal.
 */
 
 #include "linalg.h"
+#include "modcontext.h"
 #include <vector>
 #include <Eigen/Geometry>
 
 class Camera
 {
 public:
-	enum CameraType { NONE, GIMBAL, VECTOR } type;
-	enum ProjectionType { ORTHOGONAL, PERSPECTIVE } projection;
-	Camera(enum CameraType camtype = NONE);
+	enum class ProjectionType { ORTHOGONAL, PERSPECTIVE } projection;
+	Camera();
 	void setup(std::vector<double> params);
 	void gimbalDefaultTranslate();
 	void setProjection(ProjectionType type);
-	void zoom(int delta);
-	double zoomValue();
+	void zoom(int delta, bool relative);
+	double zoomValue() const;
+	double fovValue() const;
 	void resetView();
+	void updateView(const std::shared_ptr<class FileContext> ctx, bool enableWarning);
 	void viewAll(const BoundingBox &bbox);
-	std::string statusText();
+	std::string statusText() const;
 
-	// Vectorcam
-	Eigen::Vector3d eye;
-	Eigen::Vector3d center; // (aka 'target')
-	Eigen::Vector3d up; // not used currently
+	// accessors to get and set camera settings in the user space format (different for historical reasons)
+	Eigen::Vector3d getVpt() const;
+	void setVpt(double x, double y, double z);
+	Eigen::Vector3d getVpr() const;
+	void setVpr(double x, double y, double z);
+	void setVpd(double d);
+	void setVpf(double d);
 
 	// Gimbalcam
 	Eigen::Vector3d object_trans;
@@ -58,9 +63,11 @@ public:
 	unsigned int pixel_width;
 	unsigned int pixel_height;
 
+	// true if camera settings are fixed
+	// (--camera option in commandline mode)
+	bool locked;
+
 protected:
-        // Perspective settings
+	// Perspective settings
 	double viewer_distance;
-	// Orthographic settings
-	double height; // world-space height of viewport
 };
