@@ -23,6 +23,7 @@ public:
 	Response visit(State &state, const AbstractPolyNode &node) override;
 	Response visit(State &state, const LinearExtrudeNode &node) override;
 	Response visit(State &state, const RotateExtrudeNode &node) override;
+	Response visit(State &state, const ListNode &node) override;
 	Response visit(State &state, const GroupNode &node) override;
 	Response visit(State &state, const RootNode &node) override;
 	Response visit(State &state, const LeafNode &node) override;
@@ -39,6 +40,8 @@ public:
 private:
 	class ResultObject {
 	public:
+		// Default constructor with nullptr can be used to represent empty geometry,
+		// for example union() with no children, etc.
 		ResultObject() : is_const(true) {}
 		ResultObject(const Geometry *g) : is_const(true), const_pointer(g) {}
 		ResultObject(shared_ptr<const Geometry> &g) : is_const(true), const_pointer(g) {}
@@ -58,6 +61,7 @@ private:
 	void smartCacheInsert(const AbstractNode &node, const shared_ptr<const Geometry> &geom);
 	shared_ptr<const Geometry> smartCacheGet(const AbstractNode &node, bool preferNef);
 	bool isSmartCached(const AbstractNode &node);
+	bool isValidDim(const Geometry::GeometryItem &item, unsigned int &dim) const;
 	std::vector<const class Polygon2d *> collectChildren2D(const AbstractNode &node);
 	Geometry::Geometries collectChildren3D(const AbstractNode &node);
 	Polygon2d *applyMinkowski2D(const AbstractNode &node);
@@ -68,6 +72,7 @@ private:
 	ResultObject applyToChildren3D(const AbstractNode &node, OpenSCADOperator op);
 	ResultObject applyToChildren(const AbstractNode &node, OpenSCADOperator op);
 	void addToParent(const State &state, const AbstractNode &node, const shared_ptr<const Geometry> &geom);
+	Response lazyEvaluateRootNode(State &state, const AbstractNode& node);
 
 	std::map<int, Geometry::Geometries> visitedchildren;
 	const Tree &tree;
