@@ -1,5 +1,4 @@
-#ifndef OFFSCREENCONTEXT_H_
-#define OFFSCREENCONTEXT_H_
+#pragma once
 
 // Here we implement a 'portability' pattern but since we are mixing
 // Objective-C with C++, it is a bit different. The main struct
@@ -12,8 +11,6 @@
 
 struct OffscreenContext *create_offscreen_context(int w, int h);
 bool teardown_offscreen_context(OffscreenContext *ctx);
-bool save_framebuffer(OffscreenContext *ctx, const char * filename);
-bool save_framebuffer(OffscreenContext *ctx, std::ostream &output);
+bool save_framebuffer(const OffscreenContext *ctx, const char * filename);
+bool save_framebuffer(const OffscreenContext *ctx, std::ostream &output);
 std::string offscreen_context_getinfo(OffscreenContext *ctx);
-
-#endif
