@@ -29,6 +29,14 @@
 #include "polyset.h"
 #include "csgnode.h"
 
+#define OPENGL_TEST(place) \
+do { \
+	auto err = glGetError(); \
+	if (err != GL_NO_ERROR) { \
+		fprintf(stderr, "OpenGL error " place ":\n %s\n\n", gluErrorString(err)); \
+	} \
+} while (false)
+
 #ifdef ENABLE_OPENCSG
 #include <opencsg.h>
 
@@ -36,17 +44,19 @@ class OpenCSGPrim : public OpenCSG::Primitive
 {
 public:
 	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
-	OpenCSGPrim(OpenCSG::Operation operation, unsigned int convexity) :
-			OpenCSG::Primitive(operation, convexity), csgmode(Renderer::CSGMODE_NONE) { }
+	OpenCSGPrim(OpenCSG::Operation operation, unsigned int convexity, const OpenCSGRenderer &renderer) :
+			OpenCSG::Primitive(operation, convexity), csgmode(Renderer::CSGMODE_NONE), renderer(renderer) { }
 	shared_ptr<const Geometry> geom;
 	Transform3d m;
 	Renderer::csgmode_e csgmode;
 	void render() override {
 		glPushMatrix();
 		glMultMatrixd(m.data());
-		Renderer::render_surface(geom, csgmode, m);
+		renderer.render_surface(geom, csgmode, m);
 		glPopMatrix();
 	}
+private:
+	const OpenCSGRenderer &renderer;
 };
 
 #endif
@@ -54,39 +64,46 @@ public:
 OpenCSGRenderer::OpenCSGRenderer(shared_ptr<CSGProducts> root_products,
 																 shared_ptr<CSGProducts> highlights_products,
 																 shared_ptr<CSGProducts> background_products,
-																 GLint *shaderinfo)
-	: root_products(root_products), 
-		highlights_products(highlights_products), 
+																 GLView::shaderinfo_t *shaderinfo)
+	: root_products(root_products),
+		highlights_products(highlights_products),
 		background_products(background_products), shaderinfo(shaderinfo)
 {
 }
 
 void OpenCSGRenderer::draw(bool /*showfaces*/, bool showedges) const
 {
-	GLint *shaderinfo = this->shaderinfo;
-	if (!shaderinfo[0]) shaderinfo = nullptr;
+	GLView::shaderinfo_t *shaderinfo = this->shaderinfo;
+	if (!shaderinfo->progid) shaderinfo = nullptr;
+	if (!showedges) shaderinfo = nullptr;
+
+	this->draw_with_shader(shaderinfo);
+}
+
+void OpenCSGRenderer::draw_with_shader(const GLView::shaderinfo_t *shaderinfo) const
+{
 	if (this->root_products) {
-		renderCSGProducts(*this->root_products, showedges ? shaderinfo : nullptr, false, false);
+		renderCSGProducts(*this->root_products, shaderinfo, false, false);
 	}
 	if (this->background_products) {
-		renderCSGProducts(*this->background_products, showedges ? shaderinfo : nullptr, false, true);
+		renderCSGProducts(*this->background_products, shaderinfo, false, true);
 	}
 	if (this->highlights_products) {
-		renderCSGProducts(*this->highlights_products, showedges ? shaderinfo : nullptr, true, false);
+		renderCSGProducts(*this->highlights_products, shaderinfo, true, false);
 	}
 }
 
 // Primitive for rendering using OpenCSG
 OpenCSGPrim *OpenCSGRenderer::createCSGPrimitive(const CSGChainObject &csgobj, OpenCSG::Operation operation, bool highlight_mode, bool background_mode, OpenSCADOperator type) const
 {
-	OpenCSGPrim *prim = new OpenCSGPrim(operation, csgobj.leaf->geom->getConvexity());
+	OpenCSGPrim *prim = new OpenCSGPrim(operation, csgobj.leaf->geom->getConvexity(), *this);
 	prim->geom = csgobj.leaf->geom;
 	prim->m = csgobj.leaf->matrix;
     prim->csgmode = get_csgmode(highlight_mode, background_mode, type);
 	return prim;
 }
 
-void OpenCSGRenderer::renderCSGProducts(const CSGProducts &products, GLint *shaderinfo, 
+void OpenCSGRenderer::renderCSGProducts(const CSGProducts &products, const GLView::shaderinfo_t *shaderinfo,
 										bool highlight_mode, bool background_mode) const
 {
 #ifdef ENABLE_OPENCSG
@@ -102,12 +119,22 @@ void OpenCSGRenderer::renderCSGProducts(const CSGProducts &products, GLint *shad
 			OpenCSG::render(primitives);
 			glDepthFunc(GL_EQUAL);
 		}
-		if (shaderinfo) glUseProgram(shaderinfo[0]);
+		OPENGL_TEST("start");
+		if (shaderinfo) glUseProgram(shaderinfo->progid);
+		OPENGL_TEST("load shader");
+
+		for (const auto &csgobj : product.intersections) {
+			if (shaderinfo && shaderinfo->type == GLView::shaderinfo_t::SELECT_RENDERING) {
+				int identifier = csgobj.leaf->index;
+				glUniform3f(shaderinfo->data.select_rendering.identifier,
+										((identifier >> 0) & 0xff) / 255.0f, ((identifier >> 8) & 0xff) / 255.0f,
+										((identifier >> 16) & 0xff) / 255.0f);
+				OPENGL_TEST("setup shader");
+			}
 
-		for(const auto &csgobj : product.intersections) {
 			const Color4f &c = csgobj.leaf->color;
-				csgmode_e csgmode = get_csgmode(highlight_mode, background_mode);
-			
+			csgmode_e csgmode = get_csgmode(highlight_mode, background_mode);
+
 			ColorMode colormode = ColorMode::NONE;
 			if (highlight_mode) {
 				colormode = ColorMode::HIGHLIGHT;
@@ -116,10 +143,10 @@ void OpenCSGRenderer::renderCSGProducts(const CSGProducts &products, GLint *shad
 			} else {
 				colormode = ColorMode::MATERIAL;
 			}
-			
+
 			glPushMatrix();
 			glMultMatrixd(csgobj.leaf->matrix.data());
-			
+
 			const Color4f c1 = setColor(colormode, c.data(), shaderinfo);
 			if (c1[3] == 1.0f) {
 				// object is opaque, draw normally
@@ -133,13 +160,14 @@ void OpenCSGRenderer::renderCSGProducts(const CSGProducts &products, GLint *shad
 				render_surface(csgobj.leaf->geom, csgmode, csgobj.leaf->matrix, shaderinfo);
 				glDisable(GL_CULL_FACE);
 			}
+			OPENGL_TEST("render");
 
 			glPopMatrix();
 		}
 		for(const auto &csgobj : product.subtractions) {
 			const Color4f &c = csgobj.leaf->color;
 				csgmode_e csgmode = get_csgmode(highlight_mode, background_mode, OpenSCADOperator::DIFFERENCE);
-			
+
 			ColorMode colormode = ColorMode::NONE;
 			if (highlight_mode) {
 				colormode = ColorMode::HIGHLIGHT;
@@ -148,7 +176,7 @@ void OpenCSGRenderer::renderCSGProducts(const CSGProducts &products, GLint *shad
 			} else {
 				colormode = ColorMode::CUTOUT;
 			}
-			
+
 			setColor(colormode, c.data(), shaderinfo);
 			glPushMatrix();
 			glMultMatrixd(csgobj.leaf->matrix.data());
