@@ -6,17 +6,13 @@
 #include "qtgettext.h"
 #include "ui_Preferences.h"
 #include "settings.h"
+#include "initConfig.h"
 
-class Preferences : public QMainWindow, public Ui::Preferences
+class Preferences : public QMainWindow, public Ui::Preferences,public InitConfigurator
 {
 	Q_OBJECT;
 
 public:
-	static constexpr const char* PREF_EDITOR_TYPE = "editor/editortype";
-
-	static constexpr const char* EDITOR_TYPE_SIMPLE = "Simple Editor";
-	static constexpr const char* EDITOR_TYPE_QSCINTILLA = "QScintilla Editor";
-
 	~Preferences();
 	
 	static void create(QStringList colorSchemes);
@@ -24,7 +20,7 @@ public:
 	
 	QVariant getValue(const QString &key) const;
 	void init();
-	void apply() const;
+	void apply_win() const;
 	void updateGUI();
 	void fireEditorConfigChanged() const;
 
@@ -47,20 +43,21 @@ public slots:
 	void on_autoReloadRaiseCheckBox_toggled(bool);
 	void on_updateCheckBox_toggled(bool);
 	void on_snapshotCheckBox_toggled(bool);
-	void on_mdiCheckBox_toggled(bool);
 	void on_reorderCheckBox_toggled(bool);
 	void on_undockCheckBox_toggled(bool);
 	void on_checkNowButton_clicked();
 	void on_launcherBox_toggled(bool);
-	void on_editorType_currentIndexChanged(int);
 	void on_enableSoundOnRenderCompleteCheckBox_toggled(bool);
 	void on_enableHardwarningsCheckBox_toggled(bool);
 	void on_enableParameterCheckBox_toggled(bool);
 	void on_enableRangeCheckBox_toggled(bool);
+	void on_useAsciiSTLCheckBox_toggled(bool);
 	void on_enableHidapiTraceCheckBox_toggled(bool);
 	void on_checkBoxShowWarningsIn3dView_toggled(bool);
 	void on_checkBoxMouseCentricZoom_toggled(bool);
 	void on_timeThresholdOnRenderCompleteSoundEdit_textChanged(const QString &);
+	void on_checkBoxEnableAutocomplete_toggled(bool);
+	void on_lineEditCharacterThreshold_textChanged(const QString &);
   //
 	// editor settings
   //
@@ -81,6 +78,8 @@ public slots:
 	void on_spinBoxLineWrapIndentationIndent_valueChanged(int);
 	void on_comboBoxLineWrapVisualizationStart_activated(int);
 	void on_comboBoxLineWrapVisualizationEnd_activated(int);
+	void on_comboBoxModifierNumberScrollWheel_activated(int);
+
 
 	// Display
 	void on_checkBoxHighlightCurrentLine_toggled(bool);
@@ -101,17 +100,23 @@ public slots:
 
 signals:
 	void requestRedraw() const;
-	void updateMdiMode(bool mdi) const;
 	void updateUndockMode(bool undockMode) const;
 	void updateReorderMode(bool undockMode) const;
 	void fontChanged(const QString &family, uint size) const;
 	void colorSchemeChanged(const QString &scheme) const;
 	void openCSGSettingsChanged() const;
 	void syntaxHighlightChanged(const QString &s) const;
-	void editorTypeChanged(const QString &type);
 	void editorConfigChanged() const;
 	void ExperimentalChanged() const ;
 	void updateMouseCentricZoom(bool state) const;
+	void autocompleteChanged(bool status) const;
+	void characterThresholdChanged(int val) const;
+	void stepSizeChanged(int val) const;
+
+private slots:
+    void on_lineEditStepSize_textChanged(const QString &arg1);
+
+    void on_checkBoxEnableNumberScrollWheel_toggled(bool checked);
 
 private:
     Preferences(QWidget *parent = nullptr);
@@ -124,14 +129,8 @@ private:
 	void hidePasswords();
 	void addPrefPage(QActionGroup *group, QAction *action, QWidget *widget);
 
-	/** Initialize combobox list values from the settings range values */
-	void initComboBox(QComboBox *comboBox, const Settings::SettingsEntry& entry);
-	/** Initialize spinbox min/max values from the settings range values */
-	void initSpinBox(QSpinBox *spinBox, const Settings::SettingsEntry& entry);
-	/** Update combobox from current settings */
-	void updateComboBox(QComboBox *comboBox, const Settings::SettingsEntry& entry);
 	/** Set value from combobox to settings */
-	void applyComboBox(QComboBox *comboBox, int val, Settings::SettingsEntry& entry);
+	void applyComboBox(QComboBox * comboBox, int val, Settings::SettingsEntry& entry);
 
 	QSettings::SettingsMap defaultmap;
 	QHash<const QAction *, QWidget *> prefPages;
