@@ -39,14 +39,12 @@ public:
   /*!
     Return the new element array.
   */
-  const T *getArray() {
+  const std::vector<T>& getArray() {
     this->vec.resize(this->map.size());
-    typename std::unordered_map<T, int>::const_iterator iter = this->map.begin();
-    while (iter != this->map.end()) {
-      this->vec[iter->second] = iter->first;
-      iter++;
+    for (const auto& entry : map) {
+      this->vec[entry.second] = entry.first;
     }
-    return &this->vec[0];
+    return this->vec;
   }
 
   /*!
