@@ -35,6 +35,7 @@
 #include "qtgettext.h"
 #include "PlatformUtils.h"
 #include "QSettingsCached.h"
+#include "boost-utils.h"
 
 
 #include <boost/property_tree/ptree.hpp>
@@ -58,15 +59,39 @@ QFileInfo UIUtils::openFile(QWidget *parent)
     return fileInfo;
 }
 
+QFileInfoList UIUtils::openFiles(QWidget *parent)
+{
+    QSettingsCached settings;
+    QString last_dirname = settings.value("lastOpenDirName").toString();
+    QStringList new_filenames = QFileDialog::getOpenFileNames(parent, "Open File",
+	    last_dirname, "OpenSCAD Designs (*.scad *.csg)");
+
+    QFileInfoList fileInfoList;
+    for(QString filename: new_filenames)
+    {
+		if(filename.isEmpty()) {
+			continue;
+		}
+		fileInfoList.append(QFileInfo(filename));
+    }
+
+    if(!fileInfoList.isEmpty())
+    {
+	    QDir last_dir = fileInfoList[fileInfoList.size() - 1].dir(); // last_dir is set to directory of last choosen valid file
+	    last_dirname = last_dir.path();
+	    settings.setValue("lastOpenDirName", last_dirname);
+	}
+
+    return fileInfoList;
+}
+
 QStringList UIUtils::recentFiles()
 {
     QSettingsCached settings; // set up project and program properly in main.cpp
     QStringList files = settings.value("recentFileList").toStringList();
 
     // Remove any duplicate or empty entries from the list
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
     files.removeDuplicates();
-#endif
     files.removeAll(QString());
     // Now remove any entries which do not exist
     for (int i = files.size() - 1; i >= 0; --i) {
@@ -94,7 +119,7 @@ static ptree *examplesTree()
 			examples_tree = new ptree;
 			read_json(path, *examples_tree);
 		} catch (const std::exception & e) {
-			PRINTB("Error reading examples.json: %s", e.what());
+			LOG(message_group::None,Location::NONE,"","Error reading examples.json: %1$s",e.what());
 			delete examples_tree;
 			examples_tree = nullptr;
 		}
@@ -131,6 +156,11 @@ QFileInfoList UIUtils::exampleFiles(const QString &category)
 	return examples;
 }
 
+void UIUtils::openURL(const QString& url)
+{
+    QDesktopServices::openUrl(QUrl(url));
+}
+
 void UIUtils::openHomepageURL()
 {
     QDesktopServices::openUrl(QUrl("https://www.openscad.org/"));
