@@ -40,56 +40,55 @@ class CgaladvModule : public AbstractModule
 public:
 	CgaladvType type;
 	CgaladvModule(CgaladvType type) : type(type) { }
-	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
+	AbstractNode *instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const override;
 };
 
-AbstractNode *CgaladvModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
+AbstractNode *CgaladvModule::instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const
 {
-	auto node = new CgaladvNode(inst, type);
+	auto node = new CgaladvNode(inst, evalctx, type);
 
 	AssignmentList args;
 
-	if (type == CgaladvType::MINKOWSKI)
-		args += Assignment("convexity");
-
-	if (type == CgaladvType::RESIZE)
-		args += Assignment("newsize"), Assignment("auto");
-
-	Context c(ctx);
-	c.setVariables(evalctx, args);
-	inst->scope.apply(*evalctx);
-
-	auto convexity = ValuePtr::undefined;
-	auto path = ValuePtr::undefined;
-	
 	if (type == CgaladvType::MINKOWSKI) {
-		convexity = c.lookup_variable("convexity", true);
+		args += assignment("convexity");
 	}
 
 	if (type == CgaladvType::RESIZE) {
-		auto ns = c.lookup_variable("newsize");
-		node->newsize << 0,0,0;
-		if ( ns->type() == Value::ValueType::VECTOR ) {
-			const Value::VectorType &vs = ns->toVector();
-			if ( vs.size() >= 1 ) node->newsize[0] = vs[0]->toDouble();
-			if ( vs.size() >= 2 ) node->newsize[1] = vs[1]->toDouble();
-			if ( vs.size() >= 3 ) node->newsize[2] = vs[2]->toDouble();
-		}
-		auto autosize = c.lookup_variable("auto");
-		node->autosize << false, false, false;
-		if ( autosize->type() == Value::ValueType::VECTOR ) {
-			const Value::VectorType &va = autosize->toVector();
-			if ( va.size() >= 1 ) node->autosize[0] = va[0]->toBool();
-			if ( va.size() >= 2 ) node->autosize[1] = va[1]->toBool();
-			if ( va.size() >= 3 ) node->autosize[2] = va[2]->toBool();
-		}
-		else if ( autosize->type() == Value::ValueType::BOOL ) {
-			node->autosize << autosize->toBool(),autosize->toBool(),autosize->toBool();
-		}
+		args += assignment("newsize"), assignment("auto"), assignment("convexity");
 	}
 
-	node->convexity = static_cast<int>(convexity->toDouble());
-	node->path = path;
+	ContextHandle<Context> c{Context::create<Context>(ctx)};
+	c->setVariables(evalctx, args);
+	inst->scope.apply(evalctx);
+
+	if (type == CgaladvType::MINKOWSKI) {
+		const auto &convexity = c->lookup_variable("convexity", true);
+		node->convexity = static_cast<int>(convexity.toDouble());
+	} else if (type == CgaladvType::RESIZE) {
+		const auto &convexity = c->lookup_variable("convexity", true);
+		node->convexity = static_cast<int>(convexity.toDouble());
+		const auto &ns = c->lookup_variable("newsize");
+		node->newsize << 0,0,0;
+		if ( ns.type() == Value::Type::VECTOR ) {
+			const auto &vs = ns.toVector();
+			if ( vs.size() >= 1 ) node->newsize[0] = vs[0].toDouble();
+			if ( vs.size() >= 2 ) node->newsize[1] = vs[1].toDouble();
+			if ( vs.size() >= 3 ) node->newsize[2] = vs[2].toDouble();
+		}
+		const auto &autosize = c->lookup_variable("auto");
+		node->autosize << false, false, false;
+		if (autosize.type() == Value::Type::VECTOR) {
+			const auto &va = autosize.toVector();
+			if ( va.size() >= 1 ) node->autosize[0] = va[0].toBool();
+			if ( va.size() >= 2 ) node->autosize[1] = va[1].toBool();
+			if ( va.size() >= 3 ) node->autosize[2] = va[2].toBool();
+		}
+		else if ( autosize.type() == Value::Type::BOOL ) {
+			node->autosize << autosize.toBool(),autosize.toBool(),autosize.toBool();
+		}
+	} else {
+		node->convexity = 0;
+	}
 
 	auto instantiatednodes = inst->instantiateChildren(evalctx);
 	node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
@@ -132,6 +131,7 @@ std::string CgaladvNode::toString() const
 		  << this->newsize[0] << "," << this->newsize[1] << "," << this->newsize[2] << "]"
 		  << ", auto = ["
 		  << this->autosize[0] << "," << this->autosize[1] << "," << this->autosize[2] << "]"
+		  << ", convexity = " << this->convexity
 		  << ")";
 		break;
 	default:
@@ -143,7 +143,21 @@ std::string CgaladvNode::toString() const
 
 void register_builtin_cgaladv()
 {
-	Builtins::init("minkowski", new CgaladvModule(CgaladvType::MINKOWSKI));
-	Builtins::init("hull", new CgaladvModule(CgaladvType::HULL));
-	Builtins::init("resize", new CgaladvModule(CgaladvType::RESIZE));
+	Builtins::init("minkowski", new CgaladvModule(CgaladvType::MINKOWSKI),
+				{
+					"minkowski(convexity = number)",
+				});
+
+	Builtins::init("hull", new CgaladvModule(CgaladvType::HULL),
+				{
+					"hull()",
+				});
+
+	Builtins::init("resize", new CgaladvModule(CgaladvType::RESIZE),
+				{
+					"resize([x, y, z])",
+					"resize([x, y, z], boolean)",
+					"resize([x, y, z], [boolean, boolean, boolean])",
+					"resize([x, y, z], [boolean, boolean, boolean], convexity = number)",
+				});
 }
