@@ -14,14 +14,13 @@ class CgaladvNode : public AbstractNode
 {
 public:
 	VISITABLE();
-	CgaladvNode(const ModuleInstantiation *mi, CgaladvType type) : AbstractNode(mi), type(type) {
+	CgaladvNode(const ModuleInstantiation *mi, const std::shared_ptr<EvalContext> &ctx, CgaladvType type) : AbstractNode(mi, ctx), type(type) {
 		convexity = 1;
 	}
 	~CgaladvNode() { }
 	std::string toString() const override;
 	std::string name() const override;
 
-	ValuePtr path;
 	unsigned int convexity;
 	Vector3d newsize;
 	Eigen::Matrix<bool,3,1> autosize;
