@@ -45,7 +45,7 @@ class ColorModule : public AbstractModule
 public:
 	ColorModule();
 	~ColorModule();
-	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
+	AbstractNode *instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const override;
 
 private:
 	static std::unordered_map<std::string, Color4f> webcolors;
@@ -244,7 +244,7 @@ boost::optional<Color4f> parse_hex_color(const std::string& hex) {
 	Color4f rgba;
 	rgba[3] = 1.0; // default alpha to 100%
 
-	for (unsigned i = 0; i < (hex.size() - 1) / stride; i++) {
+	for (unsigned i = 0; i < (hex.size() - 1) / stride; ++i) {
 		const std::string chunk = hex.substr(1 + i*stride, stride);
 
 		// convert the hex character(s) from base 16 to base 10
@@ -254,26 +254,27 @@ boost::optional<Color4f> parse_hex_color(const std::string& hex) {
 	return rgba;
 }
 
-AbstractNode *ColorModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
+AbstractNode *ColorModule::instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const
 {
-	auto node = new ColorNode(inst);
+	auto node = new ColorNode(inst, evalctx);
 
-	AssignmentList args{Assignment("c"), Assignment("alpha")};
+	AssignmentList args{assignment("c"), assignment("alpha")};
 
-	Context c(ctx);
-	c.setVariables(evalctx, args);
-	inst->scope.apply(*evalctx);
+	ContextHandle<Context> c{Context::create<Context>(ctx)};
+	c->setVariables(evalctx, args);
+	inst->scope.apply(evalctx);
 
-	auto v = c.lookup_variable("c");
-	if (v->type() == Value::ValueType::VECTOR) {
-		for (size_t i = 0; i < 4; i++) {
-			node->color[i] = i < v->toVector().size() ? (float)v->toVector()[i]->toDouble() : 1.0f;
+	const auto &v = c->lookup_variable("c");
+	if (v.type() == Value::Type::VECTOR) {
+		const auto &vec = v.toVector();
+		for (size_t i = 0; i < 4; ++i) {
+			node->color[i] = i < v.toVector().size() ? (float)v.toVector()[i].toDouble() : 1.0f;
 			if (node->color[i] > 1 || node->color[i] < 0){
-				PRINTB_NOCACHE("WARNING: color() expects numbers between 0.0 and 1.0. Value of %.1f is out of range, %s", node->color[i] % inst->location().toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),"color() expects numbers between 0.0 and 1.0. Value of %1$.1f is out of range",node->color[i]);
 			}
 		}
-	} else if (v->type() == Value::ValueType::STRING) {
-		auto colorname = v->toString();
+	} else if (v.type() == Value::Type::STRING) {
+		auto colorname = v.toString();
 		boost::algorithm::to_lower(colorname);
 		if (webcolors.find(colorname) != webcolors.end())	{
 			node->color = webcolors.at(colorname);
@@ -283,14 +284,14 @@ AbstractNode *ColorModule::instantiate(const Context *ctx, const ModuleInstantia
 			if (hexColor) {
 				node->color = *hexColor;
 			} else {
-				PRINTB_NOCACHE("WARNING: Unable to parse color \"%s\", %s. ", colorname % inst->location().toRelativeString(ctx->documentPath()));
-				PRINT_NOCACHE("WARNING: Please see https://en.wikipedia.org/wiki/Web_colors");
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),"Unable to parse color \"%1$s\"",colorname);
+				LOG(message_group::None,Location::NONE,"","Please see https://en.wikipedia.org/wiki/Web_colors");
 			}
 		}
 	}
-	auto alpha = c.lookup_variable("alpha");
-	if (alpha->type() == Value::ValueType::NUMBER) {
-		node->color[3] = alpha->toDouble();
+	const auto &alpha = c->lookup_variable("alpha");
+	if (alpha.type() == Value::Type::NUMBER) {
+		node->color[3] = alpha.toDouble();
 	}
 
 	auto instantiatednodes = inst->instantiateChildren(evalctx);
@@ -311,5 +312,11 @@ std::string ColorNode::name() const
 
 void register_builtin_color()
 {
-	Builtins::init("color", new ColorModule());
+	Builtins::init("color", new ColorModule(),
+				{
+					"color(c = [r, g, b, a])",
+					"color(c = [r, g, b], alpha = 1.0)",
+					"color(\"#hexvalue\")",
+					"color(\"colorname\", 1.0)",
+				});
 }
