@@ -40,11 +40,11 @@
 #include <cstdint>
 
 #include <boost/filesystem.hpp>
-std::unordered_map<std::string, ValuePtr> dxf_dim_cache;
-std::unordered_map<std::string, ValuePtr> dxf_cross_cache;
+ValueMap dxf_dim_cache;
+ValueMap dxf_cross_cache;
 namespace fs = boost::filesystem;
 
-ValuePtr builtin_dxf_dim(const Context *ctx, const EvalContext *evalctx)
+Value builtin_dxf_dim(const std::shared_ptr<Context> ctx, const std::shared_ptr<EvalContext> evalctx)
 {
 	std::string rawFilename;
 	std::string filename;
@@ -57,27 +57,27 @@ ValuePtr builtin_dxf_dim(const Context *ctx, const EvalContext *evalctx)
 	// FIXME: We don't lookup the file relative to where this function was instantiated
 	// since the path is only available for ModuleInstantiations, not function expressions.
 	// See issue #217
-	for (size_t i = 0; i < evalctx->numArgs(); i++) {
-		ValuePtr n = evalctx->getArgName(i);
-		ValuePtr v = evalctx->getArgValue(i);
-		if (evalctx->getArgName(i) == "file") {
-			rawFilename = v->toString();
+	for (size_t i = 0; i < evalctx->numArgs(); ++i) {
+		const std::string &n = evalctx->getArgName(i);
+		Value v = evalctx->getArgValue(i);
+		if (n == "file") {
+			rawFilename = v.toString();
 			filename = lookup_file(rawFilename, 
 			evalctx->documentPath(), ctx->documentPath());
 		}else if (n == "layer") {
-			layername = v->toString();
+			layername = v.toString();
 		}else if (n == "origin"){
-			bool originOk = v->getVec2(xorigin, yorigin);
+			bool originOk = v.getVec2(xorigin, yorigin);
 			originOk &= std::isfinite(xorigin) && std::isfinite(yorigin);
 			if(!originOk){
-				PRINTB("WARNING: dxf_dim(..., origin=%s) could not be converted, %s", v->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"dxf_dim(..., origin=%1$s) could not be converted",v.toEchoString());
 			}
 		}else if (n == "scale"){
-			v->getDouble(scale);
+			v.getDouble(scale);
 		} else if (n == "name") {
-			name = v->toString();
+			name = v.toString();
 		}else{
-			PRINTB("WARNING: dxf_dim(..., %s=...) is not supported, %s", n->toString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+			LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"dxf_dim(..., %1$s=...) is not supported",n);
 		}
 	}
 
@@ -90,19 +90,19 @@ ValuePtr builtin_dxf_dim(const Context *ctx, const EvalContext *evalctx)
 			lastwritetime = fs::last_write_time(filepath);
 		}
 	}else{
-		PRINTB("WARNING: Can't open DXF file '%s'! %s",
-					 rawFilename % evalctx->loc.toRelativeString(ctx->documentPath()));
-		return ValuePtr::undefined;
+		LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"Can't open DXF file '%1$s'!",rawFilename);
+		return Value::undefined.clone();
 	}
 	std::string key = STR(filename << "|" << layername << "|" << name << "|" << xorigin
 												<< "|" << yorigin <<"|" << scale << "|" << lastwritetime
 												<< "|" << filesize);
-	if (dxf_dim_cache.find(key) != dxf_dim_cache.end())
-		return dxf_dim_cache.find(key)->second;
+	auto result = dxf_dim_cache.find(key);
+	if (result != dxf_dim_cache.end())
+		return result->second.clone();
 	handle_dep(filepath.string());
 	DxfData dxf(36, 0, 0, filename, layername, xorigin, yorigin, scale);
 
-	for (size_t i = 0; i < dxf.dims.size(); i++)
+	for (size_t i = 0; i < dxf.dims.size(); ++i)
 	{
 		if (!name.empty() && dxf.dims[i].name != name)
 			continue;
@@ -116,46 +116,44 @@ ValuePtr builtin_dxf_dim(const Context *ctx, const EvalContext *evalctx)
 			double y = d->coords[4][1] - d->coords[3][1];
 			double angle = d->angle;
 			double distance_projected_on_line = std::fabs(x * cos_degrees(angle) + y * sin_degrees(angle));
-			return dxf_dim_cache[key] = ValuePtr(distance_projected_on_line);
+			return dxf_dim_cache.emplace(key, distance_projected_on_line).first->second.clone();
 		}
 		else if (type == 1) {
 			// Aligned
 			double x = d->coords[4][0] - d->coords[3][0];
 			double y = d->coords[4][1] - d->coords[3][1];
-			return dxf_dim_cache[key] = ValuePtr(sqrt(x*x + y*y));
+			return dxf_dim_cache.emplace(key, sqrt(x*x + y*y)).first->second.clone();
 		}
 		else if (type == 2) {
 			// Angular
 			double a1 = atan2_degrees(d->coords[0][0] - d->coords[5][0], d->coords[0][1] - d->coords[5][1]);
 			double a2 = atan2_degrees(d->coords[4][0] - d->coords[3][0], d->coords[4][1] - d->coords[3][1]);
-			return dxf_dim_cache[key] = ValuePtr(std::fabs(a1 - a2));
+			return dxf_dim_cache.emplace(key, std::fabs(a1 - a2)).first->second.clone();
 		}
 		else if (type == 3 || type == 4) {
 			// Diameter or Radius
 			double x = d->coords[5][0] - d->coords[0][0];
 			double y = d->coords[5][1] - d->coords[0][1];
-			return dxf_dim_cache[key] = ValuePtr(sqrt(x*x + y*y));
+			return dxf_dim_cache.emplace(key, sqrt(x*x + y*y)).first->second.clone();
 		}
 		else if (type == 5) {
 			// Angular 3 Point
 		}
 		else if (type == 6) {
 			// Ordinate
-			return dxf_dim_cache[key] = ValuePtr((d->type & 64) ? d->coords[3][0] : d->coords[3][1]);
+			return dxf_dim_cache.emplace(key, (d->type & 64) ? d->coords[3][0] : d->coords[3][1]).first->second.clone();
 		}
 
-		PRINTB("WARNING: Dimension '%s' in '%s', layer '%s' has unsupported type! %s", 
-					 name % rawFilename  % layername % evalctx->loc.toRelativeString(ctx->documentPath()));
-		return ValuePtr::undefined;
+		LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"Dimension '%1$s' in '%2$s', layer '%3$s' has unsupported type!",name,rawFilename,layername);
+		return Value::undefined.clone();
 	}
 
-	PRINTB("WARNING: Can't find dimension '%s' in '%s', layer '%s'! %s",
-				 name % rawFilename % layername % evalctx->loc.toRelativeString(ctx->documentPath()));
+	LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"Can't find dimension '%1$s' in '%2$s', layer '%3$s'!",name,rawFilename,layername);
 
-	return ValuePtr::undefined;
+	return Value::undefined.clone();
 }
 
-ValuePtr builtin_dxf_cross(const Context *ctx, const EvalContext *evalctx)
+Value builtin_dxf_cross(const std::shared_ptr<Context> ctx, const std::shared_ptr<EvalContext> evalctx)
 {
 	std::string filename;
 	std::string rawFilename;
@@ -167,24 +165,24 @@ ValuePtr builtin_dxf_cross(const Context *ctx, const EvalContext *evalctx)
 	// FIXME: We don't lookup the file relative to where this function was instantiated
 	// since the path is only available for ModuleInstantiations, not function expressions.
 	// See issue #217
-	for (size_t i = 0; i < evalctx->numArgs(); i++) {
-		ValuePtr n = evalctx->getArgName(i);
-		ValuePtr v = evalctx->getArgValue(i);
+	for (size_t i = 0; i < evalctx->numArgs(); ++i) {
+		const std::string &n = evalctx->getArgName(i);
+		Value v = evalctx->getArgValue(i);
 		if (n == "file"){
-			rawFilename = v->toString();
+			rawFilename = v.toString();
 			filename = ctx->getAbsolutePath(rawFilename);
 		}else if (n == "layer"){
-			layername = v->toString();
+			layername = v.toString();
 		}else if (n == "origin"){
-			bool originOk = v->getVec2(xorigin, yorigin);
+			bool originOk = v.getVec2(xorigin, yorigin);
 			originOk &= std::isfinite(xorigin) && std::isfinite(yorigin);
 			if(!originOk){
-				PRINTB("WARNING: dxf_cross(..., origin=%s) could not be converted, %s", v->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"dxf_cross(..., origin=%1$s) could not be converted",v.toEchoString());
 			}
 		}else if (n == "scale"){
-			v->getDouble(scale);
+			v.getDouble(scale);
 		}else{
-			PRINTB("WARNING: dxf_cross(..., %s=...) is not supported, %s", n->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+			LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"dxf_cross(..., %1$s=...) is not supported",n);
 		}
 	}
 
@@ -197,9 +195,8 @@ ValuePtr builtin_dxf_cross(const Context *ctx, const EvalContext *evalctx)
 			lastwritetime = fs::last_write_time(filepath);
 		}
 	}else{
-		PRINTB("WARNING: Can't open DXF file '%s'! %s",
-					 rawFilename % evalctx->loc.toRelativeString(ctx->documentPath()));
-		return ValuePtr::undefined;
+		LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"Can't open DXF file '%1$s'!",rawFilename);
+		return Value::undefined.clone();
 	}
 
 	std::string key = STR(filename << "|" << layername << "|" << xorigin << "|" << yorigin
@@ -207,14 +204,14 @@ ValuePtr builtin_dxf_cross(const Context *ctx, const EvalContext *evalctx)
 												<< "|" << filesize);
 
 	if (dxf_cross_cache.find(key) != dxf_cross_cache.end()) {
-		return dxf_cross_cache.find(key)->second;
+		return dxf_cross_cache.find(key)->second.clone();
 	}
 	handle_dep(filepath.string());
 	DxfData dxf(36, 0, 0, filename, layername, xorigin, yorigin, scale);
 
 	double coords[4][2];
 
-	for (size_t i = 0, j = 0; i < dxf.paths.size(); i++) {
+	for (size_t i = 0, j = 0; i < dxf.paths.size(); ++i) {
 		if (dxf.paths[i].indices.size() != 2)
 			continue;
 		coords[j][0] = dxf.points[dxf.paths[i].indices[0]][0];
@@ -234,20 +231,28 @@ ValuePtr builtin_dxf_cross(const Context *ctx, const EvalContext *evalctx)
 			// double ub = ((x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3)) / dem;
 			double x = x1 + ua*(x2 - x1);
 			double y = y1 + ua*(y2 - y1);
-			Value::VectorType ret;
-			ret.push_back(ValuePtr(x));
-			ret.push_back(ValuePtr(y));
-			return dxf_cross_cache[key] = ValuePtr(ret);
+			VectorType ret;
+			ret.emplace_back(x);
+			ret.emplace_back(y);
+			Value val(std::move(ret));
+			dxf_cross_cache.insert_or_assign(key, val.clone());
+			return val;
 		}
 	}
 
-	PRINTB("WARNING: Can't find cross in '%s', layer '%s'! %s", rawFilename % layername % evalctx->loc.toRelativeString(ctx->documentPath()));
-
-	return ValuePtr::undefined;
+	LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"Can't find cross in '%1$s', layer '%2$s'!",rawFilename,layername);
+	return Value::undefined.clone();
 }
 
 void initialize_builtin_dxf_dim()
 {
-	Builtins::init("dxf_dim", new BuiltinFunction(&builtin_dxf_dim));
-	Builtins::init("dxf_cross", new BuiltinFunction(&builtin_dxf_cross));
+	Builtins::init("dxf_dim", new BuiltinFunction(&builtin_dxf_dim),
+				{
+					"dxf_dim()",
+				});
+
+	Builtins::init("dxf_cross", new BuiltinFunction(&builtin_dxf_cross),
+				{
+					"dxf_cross()",
+				});
 }
