@@ -1,15 +1,14 @@
 #include "editor.h"
 #include "Preferences.h"
+#include "QSettingsCached.h"
 
 void EditorInterface::wheelEvent(QWheelEvent *event)
 {
-	QSettings settings;
+	QSettingsCached settings;
 	bool wheelzoom_enabled = Preferences::inst()->getValue("editor/ctrlmousewheelzoom").toBool();
 	if ((event->modifiers() == Qt::ControlModifier) && wheelzoom_enabled) {
-		if (event->delta() > 0)
-			zoomIn();
-		else if (event->delta() < 0)
-			zoomOut();
+		if (event->delta() > 0) zoomIn();
+		else if (event->delta() < 0) zoomOut();
 	} else {
 		QWidget::wheelEvent(event);
 	}
