@@ -1,11 +1,10 @@
 #pragma once
 
-#include "AST.h"
-
 #include <string>
 #include <vector>
 #include "value.h"
 #include "memory.h"
+#include "boost-utils.h"
 #include "Assignment.h"
 
 class Expression : public ASTNode
@@ -14,7 +13,8 @@ public:
 	Expression(const Location &loc) : ASTNode(loc) {}
 	~Expression() {}
 	virtual bool isLiteral() const;
-	virtual ValuePtr evaluate(const class Context *context) const = 0;
+	virtual Value evaluate(const std::shared_ptr<Context>& context) const = 0;
+	Value checkUndef(Value&& val, const std::shared_ptr<Context>& context) const;
 };
 
 class UnaryOp : public Expression
@@ -26,7 +26,7 @@ public:
 	};
 	bool isLiteral() const override;
 	UnaryOp(Op op, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 
 private:
@@ -42,6 +42,7 @@ public:
 	enum class Op {
 		LogicalAnd,
 		LogicalOr,
+		Exponent,
 		Multiply,
 		Divide,
 		Modulo,
@@ -56,7 +57,7 @@ public:
 	};
 
 	BinaryOp(Expression *left, Op op, Expression *right, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 
 private:
@@ -71,9 +72,10 @@ class TernaryOp : public Expression
 {
 public:
 	TernaryOp(Expression *cond, Expression *ifexpr, Expression *elseexpr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	const shared_ptr<Expression>& evaluateStep(const std::shared_ptr<Context>& context) const;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
-
+private:
 	shared_ptr<Expression> cond;
 	shared_ptr<Expression> ifexpr;
 	shared_ptr<Expression> elseexpr;
@@ -83,7 +85,7 @@ class ArrayLookup : public Expression
 {
 public:
 	ArrayLookup(Expression *array, Expression *index, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	shared_ptr<Expression> array;
@@ -93,12 +95,12 @@ private:
 class Literal : public Expression
 {
 public:
-	Literal(const ValuePtr &val, const Location &loc = Location::NONE);
-	ValuePtr evaluate(const class Context *) const override;
+	Literal(Value val, const Location &loc = Location::NONE);
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 	bool isLiteral() const override { return true;}
 private:
-	ValuePtr value;
+	const Value value;
 };
 
 class Range : public Expression
@@ -106,7 +108,7 @@ class Range : public Expression
 public:
 	Range(Expression *begin, Expression *end, const Location &loc);
 	Range(Expression *begin, Expression *step, Expression *end, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 	bool isLiteral() const override;
 private:
@@ -119,21 +121,23 @@ class Vector : public Expression
 {
 public:
 	Vector(const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
-	void push_back(Expression *expr);
+	void emplace_back(Expression *expr);
 	bool isLiteral() const override;
 private:
 	std::vector<shared_ptr<Expression>> children;
+	mutable boost::tribool literal_flag; // cache if already computed
 };
 
 class Lookup : public Expression
 {
 public:
 	Lookup(const std::string &name, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
-	ValuePtr evaluateSilently(const class Context *context) const;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
+	const Value& evaluateSilently(const std::shared_ptr<Context>& context) const;
 	void print(std::ostream &stream, const std::string &indent) const override;
+	const std::string& get_name() const { return name; }
 private:
 	std::string name;
 };
@@ -142,7 +146,7 @@ class MemberLookup : public Expression
 {
 public:
 	MemberLookup(Expression *expr, const std::string &member, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	shared_ptr<Expression> expr;
@@ -152,20 +156,40 @@ private:
 class FunctionCall : public Expression
 {
 public:
-	FunctionCall(const std::string &funcname, const AssignmentList &arglist, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	FunctionCall(Expression *expr, const AssignmentList &arglist, const Location &loc);
+	void prepareTailCallContext(const std::shared_ptr<Context> context, std::shared_ptr<Context> tailCallContext, const AssignmentList &definition_arguments);
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
+	const std::string& get_name() const { return name; }
 	static Expression * create(const std::string &funcname, const AssignmentList &arglist, Expression *expr, const Location &loc);
+	shared_ptr<class FunctionDefinition> getFunctionDefinition(const Value& v) const;
 public:
+	bool isLookup;
 	std::string name;
+	shared_ptr<Expression> expr;
 	AssignmentList arguments;
+	AssignmentMap resolvedArguments;
+	std::vector<std::pair<std::string, Value>> defaultArguments; // Only the ones not mentioned in 'resolvedArguments'
+};
+
+class FunctionDefinition : public Expression
+{
+public:
+	FunctionDefinition(Expression *expr, const AssignmentList &definition_arguments, const Location &loc);
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
+	void print(std::ostream &stream, const std::string &indent) const override;
+public:
+	shared_ptr<Context> ctx;
+	AssignmentList definition_arguments;
+	shared_ptr<Expression> expr;
 };
 
 class Assert : public Expression
 {
 public:
 	Assert(const AssignmentList &args, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	const shared_ptr<Expression>& evaluateStep(const std::shared_ptr<Context>& context) const;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	AssignmentList arguments;
@@ -176,7 +200,8 @@ class Echo : public Expression
 {
 public:
 	Echo(const AssignmentList &args, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	const shared_ptr<Expression>& evaluateStep(const std::shared_ptr<Context>& context) const;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	AssignmentList arguments;
@@ -187,7 +212,8 @@ class Let : public Expression
 {
 public:
 	Let(const AssignmentList &args, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	const shared_ptr<Expression>& evaluateStep(const std::shared_ptr<Context>& context) const;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	AssignmentList arguments;
@@ -205,7 +231,7 @@ class LcIf : public ListComprehension
 {
 public:
 	LcIf(Expression *cond, Expression *ifexpr, Expression *elseexpr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	shared_ptr<Expression> cond;
@@ -217,7 +243,7 @@ class LcFor : public ListComprehension
 {
 public:
 	LcFor(const AssignmentList &args, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	AssignmentList arguments;
@@ -228,7 +254,7 @@ class LcForC : public ListComprehension
 {
 public:
 	LcForC(const AssignmentList &args, const AssignmentList &incrargs, Expression *cond, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	AssignmentList arguments;
@@ -241,9 +267,10 @@ class LcEach : public ListComprehension
 {
 public:
 	LcEach(Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
+	Value evalRecur(Value &&v, const std::shared_ptr<Context>& context) const;
 	shared_ptr<Expression> expr;
 };
 
@@ -251,11 +278,16 @@ class LcLet : public ListComprehension
 {
 public:
 	LcLet(const AssignmentList &args, Expression *expr, const Location &loc);
-	ValuePtr evaluate(const class Context *context) const override;
+	Value evaluate(const std::shared_ptr<Context>& context) const override;
 	void print(std::ostream &stream, const std::string &indent) const override;
 private:
 	AssignmentList arguments;
 	shared_ptr<Expression> expr;
 };
 
-void evaluate_assert(const Context &context, const class EvalContext *evalctx);
+void evaluate_assert(const std::shared_ptr<Context>& context, const std::shared_ptr<class EvalContext> evalctx);
+
+Value evaluate_function(const std::string& name,
+		const std::shared_ptr<Expression>& expr, const AssignmentList& definition_arguments,
+		const std::shared_ptr<Context>& ctx, const std::shared_ptr<EvalContext>& evalctx,
+		const Location& loc);
