@@ -41,7 +41,7 @@
 #include "fileutils.h"
 #include "feature.h"
 #include "handle_dep.h"
-
+#include "boost-utils.h"
 #include <sys/types.h>
 #include <sstream>
 #include <boost/algorithm/string.hpp>
@@ -50,7 +50,6 @@ namespace fs = boost::filesystem;
 #include <boost/assign/std/vector.hpp>
 using namespace boost::assign; // bring 'operator+=()' into scope
 
-#include <boost/detail/endian.hpp>
 #include <cstdint>
 
 extern PolySet * import_amf(std::string, const Location &loc);
@@ -61,36 +60,39 @@ class ImportModule : public AbstractModule
 public:
 	ImportType type;
 	ImportModule(ImportType type = ImportType::UNKNOWN) : type(type) { }
-	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
+	AbstractNode *instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const override;
 };
 
-AbstractNode *ImportModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
+AbstractNode *ImportModule::instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const
 {
   AssignmentList args{
-    Assignment("file"), Assignment("layer"), Assignment("convexity"),
-		Assignment("origin"), Assignment("scale")
-	};
-	
-	AssignmentList optargs{
-		Assignment("width"), Assignment("height"),
-		Assignment("filename"), Assignment("layername"), Assignment("center"), Assignment("dpi")
+    assignment("file"), assignment("layer"), assignment("convexity"),
+		assignment("origin"), assignment("scale")
 	};
 
-	Context c(ctx);
-	c.setDocumentPath(evalctx->documentPath());
-	c.setVariables(evalctx, args, optargs);
+	AssignmentList optargs{
+		assignment("width"), assignment("height"),
+		assignment("filename"), assignment("layername"), assignment("center"), assignment("dpi")
+	};
+
+	ContextHandle<Context> c{Context::create<Context>(ctx)};
+	c->setDocumentPath(evalctx->documentPath());
+	c->setVariables(evalctx, args, optargs);
 #if 0 && DEBUG
 	c.dump(this, inst);
 #endif
 
-	auto v = c.lookup_variable("file", true);
-	if (v->isUndefined()) {
-		v = c.lookup_variable("filename", true);
-		if (!v->isUndefined()) {
-			printDeprecation("filename= is deprecated. Please use file=");
+	const auto &v = c->lookup_variable("file", true);
+	std::string filename;
+	if (v.isDefined()) {
+		filename = lookup_file(v.isUndefined() ? "" : v.toString(), inst->path(), ctx->documentPath());
+	} else {
+		const auto &filename_val = c->lookup_variable("filename", true);
+		if (!filename_val.isUndefined()) {
+			LOG(message_group::Deprecated,Location::NONE,"","filename= is deprecated. Please use file=");
 		}
+		filename = lookup_file(filename_val.isUndefined() ? "" : filename_val.toString(), inst->path(), ctx->documentPath());
 	}
-	const std::string filename = lookup_file(v->isUndefined() ? "" : v->toString(), inst->path(), ctx->documentPath());
 	if (!filename.empty()) handle_dep(filename);
 	ImportType actualtype = this->type;
 	if (actualtype == ImportType::UNKNOWN) {
@@ -105,58 +107,62 @@ AbstractNode *ImportModule::instantiate(const Context *ctx, const ModuleInstanti
 		else if (ext == ".svg") actualtype = ImportType::SVG;
 	}
 
-	auto node = new ImportNode(inst, actualtype);
+	auto node = new ImportNode(inst, evalctx, actualtype);
 
-	node->fn = c.lookup_variable("$fn")->toDouble();
-	node->fs = c.lookup_variable("$fs")->toDouble();
-	node->fa = c.lookup_variable("$fa")->toDouble();
+	node->fn = c->lookup_variable("$fn").toDouble();
+	node->fs = c->lookup_variable("$fs").toDouble();
+	node->fa = c->lookup_variable("$fa").toDouble();
 
 	node->filename = filename;
-	auto layerval = c.lookup_variable("layer", true);
-	if (layerval->isUndefined()) {
-		layerval = c.lookup_variable("layername", true);
-		if (!layerval->isUndefined()) {
-			printDeprecation("layername= is deprecated. Please use layer=");
+	const auto &layerval = c->lookup_variable("layer", true);
+	if (layerval.isDefined()) {
+		node->layername = layerval.toString();
+	} else {
+		const auto &layername = c->lookup_variable("layername", true);
+		if (layername.isDefined()) {
+			LOG(message_group::Deprecated,Location::NONE,"","layername= is deprecated. Please use layer=");
+			node->layername = layername.toString();
+		} else {
+			node->layername = "";
 		}
 	}
-	node->layername = layerval->isUndefined() ? ""  : layerval->toString();
-	node->convexity = (int)c.lookup_variable("convexity", true)->toDouble();
+	node->convexity = (int)c->lookup_variable("convexity", true).toDouble();
 
 	if (node->convexity <= 0) node->convexity = 1;
 
-	const auto origin = c.lookup_variable("origin", true);
+	const auto &origin = c->lookup_variable("origin", true);
 	node->origin_x = node->origin_y = 0;
-	bool originOk = origin->getVec2(node->origin_x, node->origin_y);
+	bool originOk = origin.getVec2(node->origin_x, node->origin_y);
 	originOk &= std::isfinite(node->origin_x) && std::isfinite(node->origin_y);
-	if(origin!=ValuePtr::undefined && !originOk){
-		PRINTB("WARNING: linear_extrude(..., origin=%s) could not be converted, %s", origin->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+	if(origin.isDefined() && !originOk){
+		LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"linear_extrude(..., origin=%1$s) could not be converted",origin.toEchoString());
 	}
 
-	const auto center = c.lookup_variable("center", true);
-	node->center = center->type() == Value::ValueType::BOOL ? center->toBool() : false;
+	const auto &center = c->lookup_variable("center", true);
+	node->center = center.type() == Value::Type::BOOL ? center.toBool() : false;
 
-	node->scale = c.lookup_variable("scale", true)->toDouble();
+	node->scale = c->lookup_variable("scale", true).toDouble();
 	if (node->scale <= 0) node->scale = 1;
 
 	node->dpi = ImportNode::SVG_DEFAULT_DPI;
-	const auto dpi = c.lookup_variable("dpi", true);
-	if (dpi->type() == Value::ValueType::NUMBER) {
-		double val = dpi->toDouble();
+	const auto &dpi = c->lookup_variable("dpi", true);
+	if (dpi.type() == Value::Type::NUMBER) {
+		double val = dpi.toDouble();
 		if (val < 0.001) {
-			PRINTB("WARNING: Invalid dpi value giving, using default of %f dpi. Value must be positive and >= 0.001, file %s, import() at line %d",
-					node->dpi %
-					inst->location().toRelativeString(ctx->documentPath()) %
-					inst->location().firstLine());
+		std::string filePath = boostfs_uncomplete(inst->location().filePath(),ctx->documentPath()).generic_string();
+		LOG(message_group::Warning,Location::NONE,"",
+			"Invalid dpi value giving, using default of %1$f dpi. Value must be positive and >= 0.001, file %2$s, import() at line %3$d",
+			origin.toEchoString(),filePath,filePath,inst->location().firstLine());
 		} else {
 			node->dpi = val;
 		}
 	}
 
-	auto width = c.lookup_variable("width", true);
-	auto height = c.lookup_variable("height", true);
-	node->width = (width->type() == Value::ValueType::NUMBER) ? width->toDouble() : -1;
-	node->height = (height->type() == Value::ValueType::NUMBER) ? height->toDouble() : -1;
-	
+	const auto &width = c->lookup_variable("width", true);
+	const auto &height = c->lookup_variable("height", true);
+	node->width = (width.type() == Value::Type::NUMBER) ? width.toDouble() : -1;
+	node->height = (height.type() == Value::Type::NUMBER) ? height.toDouble() : -1;
+
 	return node;
 }
 
@@ -201,7 +207,7 @@ const Geometry *ImportNode::createGeometry() const
 	}
 #endif
 	default:
-		PRINTB("ERROR: Unsupported file format while trying to import file '%s', import() at Line %d", this->filename % loc.firstLine());
+		LOG(message_group::Error,Location::NONE,"","Unsupported file format while trying to import file '%1$s', import() at line %2$d",this->filename,loc.firstLine());
 		g = new PolySet(3);
 	}
 
@@ -241,5 +247,9 @@ void register_builtin_import()
 	Builtins::init("import_stl", new ImportModule(ImportType::STL));
 	Builtins::init("import_off", new ImportModule(ImportType::OFF));
 	Builtins::init("import_dxf", new ImportModule(ImportType::DXF));
-	Builtins::init("import", new ImportModule());
+
+	Builtins::init("import", new ImportModule(),
+				{
+					"import(string, [number, [number]])",
+				});
 }
