@@ -1,6 +1,8 @@
 #include "import.h"
+#include "memory.h"
 #include "printutils.h"
 #include "AST.h"
+#include "boost-utils.h"
 
 #ifdef ENABLE_CGAL
 #include "CGAL_Nef_polyhedron.h"
@@ -17,18 +19,19 @@ CGAL_Nef_polyhedron *import_nef3(const std::string &filename, const Location &lo
 	// Open file and position at the end
 	std::ifstream f(filename.c_str(), std::ios::in | std::ios::binary);
 	if (!f.good()) {
-		PRINTB("WARNING: Can't open import file '%s', import() at line %d", filename % loc.firstLine());
+		LOG(message_group::Warning,Location::NONE,"","Can't open import file '%1$s', import() at line %2$d",filename,loc.firstLine());
 		return N;
 	}
 	
 	std::string msg="";
 	CGAL::Failure_behaviour old_behaviour = CGAL::set_error_behaviour(CGAL::THROW_EXCEPTION);
-	try{
-		N->p3.reset(new CGAL_Nef_polyhedron3);
-		f >> *(N->p3);
+	try {
+		auto nef = make_shared<CGAL_Nef_polyhedron3>();
+		f >> *nef;
+		N->p3 = nef;
 	} catch (const CGAL::Failure_exception &e) {
-		PRINTB("WARNING: Failure trying to import '%s', import() at line %d", filename % loc.firstLine());
-		PRINT(e.what());
+		LOG(message_group::Warning,Location::NONE,"","Failure trying to import '%1$s', import() at line %2$d",filename,loc.firstLine());
+		LOG(message_group::None,Location::NONE,"",e.what());
 		N = new CGAL_Nef_polyhedron;
 	}
 	CGAL::set_error_behaviour(old_behaviour);
