@@ -2,6 +2,8 @@
 
 #include <memory>
 using std::shared_ptr;
+using std::unique_ptr;
 using std::make_shared;
 using std::dynamic_pointer_cast;
 using std::static_pointer_cast;
+using std::nullptr_t;
