@@ -63,6 +63,9 @@ ParameterWidget::ParameterWidget(QWidget *parent) : QWidget(parent)
 	connect(comboBoxPreset, SIGNAL(currentIndexChanged(int)), this, SLOT(onSetChanged(int)));
 	connect(comboBoxPreset->lineEdit(), SIGNAL(editingFinished()), this, SLOT(onSetNameChanged()));
 	connect(reset, SIGNAL(clicked()), this, SLOT(resetParameter()));
+	connect(addButton, SIGNAL(clicked()), this, SLOT(onSetAdd()));
+	connect(deleteButton, SIGNAL(clicked()), this, SLOT(onSetDelete()));
+	connect(presetSaveButton, SIGNAL(clicked()), this, SLOT(onSetSaveButton()));
 
 	comboBoxPreset->setInsertPolicy(QComboBox::InsertAtCurrent);
 
@@ -142,20 +145,17 @@ void ParameterWidget::readFile(QString scadFile)
 	bool writeable = false;
 	bool readable = false;
 
-	if(exists){
+	this->setMgr->clear();
+	if (exists) {
 		readable = this->setMgr->readParameterSet(this->jsonFile);
-
 		//check whether file is writeable or not
 		if (std::fstream(this->jsonFile, std::ios::app)) writeable = true;
 	}
 
 	if(writeable || !exists){
-		connect(this->addButton, SIGNAL(clicked()), this, SLOT(onSetAdd()));
-		this->addButton->setToolTip(_("add new preset"));
-		connect(this->deleteButton, SIGNAL(clicked()), this, SLOT(onSetDelete()));
-		this->deleteButton->setToolTip(_("remove current preset"));
-		connect(this->presetSaveButton, SIGNAL(clicked()), this, SLOT(onSetSaveButton()));
-		this->presetSaveButton->setToolTip(_("save current preset"));
+		this->addButton->setToolTip(_("Add new preset"));
+		this->deleteButton->setToolTip(_("Remove current preset"));
+		this->presetSaveButton->setToolTip(_("Save current preset"));
 	}else{
 		this->addButton->setToolTip(_("JSON file read only"));
 		this->deleteButton->setToolTip(_("JSON file read only"));
@@ -174,8 +174,8 @@ void ParameterWidget::readFile(QString scadFile)
 }
 
 //Write the json file if the parameter sets are not empty.
-//This prevents creating unneccesary json filess.
-//This methode also updates the UI state (change indicator, file name, ...)
+//This prevents creating unnecessary json filess.
+//This method also updates the UI state (change indicator, file name, ...)
 void ParameterWidget::writeFileIfNotEmpty(QString scadFile)
 {
 	setFile(scadFile);
@@ -212,6 +212,7 @@ void ParameterWidget::applyParameters(FileModule *fileModule)
 
 void ParameterWidget::setComboBoxPresetForSet()
 {
+	this->lastComboboxIndex = -1;
 	this->comboBoxPreset->addItem(_("design default values"), QVariant(QString::fromStdString(_("design default values"))));
 	if (this->setMgr->isEmpty()) return;
 	for (const auto &name : this->setMgr->getParameterNames()) {
@@ -373,7 +374,7 @@ void ParameterWidget::connectWidget()
 
 			std::vector<std::string> gr;
 			gr = groupMap[groupName].parameterVector;
-			for(unsigned int i=0;i < gr.size();i++) {
+			for(unsigned int i=0; i < gr.size(); ++i) {
 				ParameterVirtualWidget * entry = CreateParameterWidget(gr[i]);
 				if(entry){
 					anyLayout->addWidget(entry);
@@ -408,7 +409,7 @@ void ParameterWidget::rebuildGroupMap(){
 	}
 
 	groupPos.clear();
-	for (unsigned int it=0; it<ParameterPos.size(); it++) {
+	for (unsigned int it=0; it<ParameterPos.size(); ++it) {
 		std::string groupName=entries[ParameterPos[it]]->groupName;
 		if (groupMap.find(groupName) == groupMap.end()) {
 			groupPos.push_back(groupName);
@@ -468,7 +469,7 @@ ParameterVirtualWidget* ParameterWidget::CreateParameterWidget(std::string param
 //reset all parameters to the default value of the design file
 void ParameterWidget::defaultParameter(){
 	for (const auto &entry : entries) {
-		entry.second->value=entry.second->defaultValue;
+		entry.second->value=entry.second->defaultValue.clone();
 	}
 }
 
@@ -482,18 +483,18 @@ void ParameterWidget::applyParameterSet(std::string setName)
 	for (pt::ptree::value_type &v : set.get()) {
 		entry_map_t::iterator entry = entries.find(v.first);
 		if (entry != entries.end()) {
-			if (entry->second->dvt == Value::ValueType::STRING) {
-				entry->second->value=ValuePtr(v.second.data());
-			} else if (entry->second->dvt == Value::ValueType::BOOL) {
-				entry->second->value = ValuePtr(v.second.get_value<bool>());
+			if (entry->second->dvt == Value::Type::STRING) {
+				entry->second->value = Value(v.second.data());
+			} else if (entry->second->dvt == Value::Type::BOOL) {
+				entry->second->value = Value(v.second.get_value<bool>());
 			} else {
 				shared_ptr<Expression> params = CommentParser::parser(v.second.data().c_str());
 				if (!params) continue;
 				
-				ModuleContext ctx;
-				ValuePtr newValue = params->evaluate(&ctx);
-				if (entry->second->dvt == newValue->type()) {
-					entry->second->value = newValue;
+				ContextHandle<Context> ctx{Context::create<Context>()};
+				Value newValue = params->evaluate(ctx.ctx);
+				if (entry->second->dvt == newValue.type()) {
+					entry->second->value = std::move(newValue);
 				}
 			}
 		}
@@ -548,7 +549,7 @@ void ParameterWidget::updateParameterSet(std::string setName, bool newSet)
 		pt::ptree iroot;
 		for (const auto &entry : entries) {
 			const auto &VariableName = entry.first;
-			const auto &VariableValue = entry.second->value->toString();
+			const auto &VariableValue = entry.second->value.toString();
 			iroot.put(VariableName, VariableValue);
 		}
 		this->setMgr->addParameterSet(setName, iroot);
