@@ -11,13 +11,11 @@ public:
 
 private:
 	double step;
-	bool pressed;
 	bool volatile suppressUpdate; 
+	static constexpr uint32_t max_uint32 = std::numeric_limits<uint32_t>::max();
 
 protected slots:
 	void onSliderChanged(int);
 	void onSpinBoxChanged(double);
-	void onReleased();
-	void onPressed();
 	void onEditingFinished();
 };
