@@ -39,6 +39,7 @@
 #include <cmath>
 #include <boost/assign/std/vector.hpp>
 #include "ModuleInstantiation.h"
+#include "boost-utils.h"
 using namespace boost::assign; // bring 'operator+=()' into scope
 
 #define F_MINIMUM 0.01
@@ -58,44 +59,30 @@ class PrimitiveModule : public AbstractModule
 public:
 	primitive_type_e type;
 	PrimitiveModule(primitive_type_e type) : type(type) { }
-	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
+	AbstractNode *instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const override;
 private:
-	Value lookup_radius(const Context &ctx, const Location &loc, const std::string &radius_var, const std::string &diameter_var) const;
+	Value lookup_radius(const std::shared_ptr<Context> ctx, const Location &loc, const std::string &radius_var, const std::string &diameter_var) const;
 };
 
 class PrimitiveNode : public LeafNode
 {
 public:
 	VISITABLE();
-	PrimitiveNode(const ModuleInstantiation *mi, primitive_type_e type, const std::string &docPath) : LeafNode(mi), document_path(docPath), type(type) { }
+	PrimitiveNode(const ModuleInstantiation *mi, const std::shared_ptr<EvalContext> &ctx, primitive_type_e type, const std::string &docPath) : LeafNode(mi, ctx),
+			document_path(docPath), type(type), points(Value::undefined.clone()), paths(Value::undefined.clone()), faces(Value::undefined.clone()) { }
 	std::string toString() const override;
 	std::string name() const override {
 		switch (this->type) {
-		case primitive_type_e::CUBE:
-			return "cube";
-			break;
-		case primitive_type_e::SPHERE:
-			return "sphere";
-			break;
-		case primitive_type_e::CYLINDER:
-			return "cylinder";
-			break;
-		case primitive_type_e::POLYHEDRON:
-			return "polyhedron";
-			break;
-		case primitive_type_e::SQUARE:
-			return "square";
-			break;
-		case primitive_type_e::CIRCLE:
-			return "circle";
-			break;
-		case primitive_type_e::POLYGON:
-			return "polygon";
-			break;
-		default:
-			assert(false && "PrimitiveNode::name(): Unknown primitive type");
-			return "unknown";
+		case primitive_type_e::CUBE:       return "cube";
+		case primitive_type_e::SPHERE:     return "sphere";
+		case primitive_type_e::CYLINDER:   return "cylinder";
+		case primitive_type_e::POLYHEDRON: return "polyhedron";
+		case primitive_type_e::SQUARE:     return "square";
+		case primitive_type_e::CIRCLE:     return "circle";
+		case primitive_type_e::POLYGON:    return "polygon";
+		default: assert(false && "PrimitiveNode::name(): Unknown primitive type");
 		}
+		return "unknown";
 	}
 	const std::string document_path;
 
@@ -104,7 +91,7 @@ public:
 	double fn, fs, fa;
 	primitive_type_e type;
 	int convexity;
-	ValuePtr points, paths, faces;
+	Value points, paths, faces;
 	const Geometry *createGeometry() const override;
 };
 
@@ -112,35 +99,35 @@ public:
  * Return a radius value by looking up both a diameter and radius variable.
  * The diameter has higher priority, so if found an additionally set radius
  * value is ignored.
- * 
+ *
  * @param ctx data context with variable values.
  * @param radius_var name of the variable to lookup for the radius value.
  * @param diameter_var name of the variable to lookup for the diameter value.
- * @return radius value of type Value::ValueType::NUMBER or Value::ValueType::UNDEFINED if both
+ * @return radius value of type Value::Type::NUMBER or Value::Type::UNDEFINED if both
  *         variables are invalid or not set.
  */
-Value PrimitiveModule::lookup_radius(const Context &ctx, const Location &loc, const std::string &diameter_var, const std::string &radius_var) const
+Value PrimitiveModule::lookup_radius(const std::shared_ptr<Context> ctx, const Location &loc, const std::string &diameter_var, const std::string &radius_var) const
 {
-	auto d = ctx.lookup_variable(diameter_var, true);
-	auto r = ctx.lookup_variable(radius_var, true);
-	const auto r_defined = (r->type() == Value::ValueType::NUMBER);
-	
-	if (d->type() == Value::ValueType::NUMBER) {
+	const auto &d = ctx->lookup_variable(diameter_var, true);
+	const auto &r = ctx->lookup_variable(radius_var, true);
+	const auto r_defined = (r.type() == Value::Type::NUMBER);
+
+	if (d.type() == Value::Type::NUMBER) {
 		if (r_defined) {
-			std::string locStr = loc.toRelativeString(ctx.documentPath());
-			PRINTB("WARNING: Ignoring radius variable '%s' as diameter '%s' is defined too, %s", radius_var % diameter_var % locStr);
+			LOG(message_group::Warning,loc,ctx->documentPath(),
+				"Ignoring radius variable '%1$s' as diameter '%2$s' is defined too.",radius_var,diameter_var);
 		}
-		return {d->toDouble() / 2.0};
+		return d.toDouble() / 2.0;
 	} else if (r_defined) {
-		return *r;
+		return r.clone();
 	} else {
-		return Value::undefined;
+		return Value::undefined.clone();
 	}
 }
 
-AbstractNode *PrimitiveModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
+AbstractNode *PrimitiveModule::instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const
 {
-	auto node = new PrimitiveNode(inst, this->type, ctx->documentPath());
+	auto node = new PrimitiveNode(inst, evalctx, this->type, ctx->documentPath());
 
 	node->center = false;
 	node->x = node->y = node->z = node->h = node->r1 = node->r2 = 1;
@@ -148,192 +135,194 @@ AbstractNode *PrimitiveModule::instantiate(const Context *ctx, const ModuleInsta
 	AssignmentList args;
 	AssignmentList optargs;
 	if(inst->scope.hasChildren()){
-		PRINTB("WARNING: module %s() does not support child modules, %s", node->name() % inst->location().toRelativeString(ctx->documentPath()));
+		LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+			"module %1$s() does not support child modules",node->name());
 	}
 
 	switch (this->type) {
 	case primitive_type_e::CUBE:
-		args += Assignment("size"), Assignment("center");
+		args += assignment("size"), assignment("center");
 		break;
 	case primitive_type_e::SPHERE:
-		args += Assignment("r");
-		optargs += Assignment("d");
+		args += assignment("r");
+		optargs += assignment("d");
 		break;
 	case primitive_type_e::CYLINDER:
-		args += Assignment("h"), Assignment("r1"), Assignment("r2"), Assignment("center");
-		optargs += Assignment("r"),  Assignment("d"), Assignment("d1"),  Assignment("d2");
+		args += assignment("h"), assignment("r1"), assignment("r2"), assignment("center");
+		optargs += assignment("r"),  assignment("d"), assignment("d1"),  assignment("d2");
 		break;
 	case primitive_type_e::POLYHEDRON:
-		args += Assignment("points"), Assignment("faces"), Assignment("convexity");
+		args += assignment("points"), assignment("faces"), assignment("convexity");
+		optargs += assignment("triangles");
 		break;
 	case primitive_type_e::SQUARE:
-		args += Assignment("size"), Assignment("center");
+		args += assignment("size"), assignment("center");
 		break;
 	case primitive_type_e::CIRCLE:
-		args += Assignment("r");
-		optargs += Assignment("d");
+		args += assignment("r");
+		optargs += assignment("d");
 		break;
 	case primitive_type_e::POLYGON:
-		args += Assignment("points"), Assignment("paths"), Assignment("convexity");
+		args += assignment("points"), assignment("paths"), assignment("convexity");
 		break;
 	default:
 		assert(false && "PrimitiveModule::instantiate(): Unknown node type");
 	}
 
-	Context c(ctx);
-	c.setVariables(evalctx, args, optargs);
+	ContextHandle<Context> c{Context::create<Context>(ctx)};
+	c->setVariables(evalctx, args, optargs);
 
-	node->fn = c.lookup_variable("$fn")->toDouble();
-	node->fs = c.lookup_variable("$fs")->toDouble();
-	node->fa = c.lookup_variable("$fa")->toDouble();
+	node->fn = c->lookup_variable("$fn").toDouble();
+	node->fs = c->lookup_variable("$fs").toDouble();
+	node->fa = c->lookup_variable("$fa").toDouble();
 
 	if (node->fs < F_MINIMUM) {
-		PRINTB("WARNING: $fs too small - clamping to %f, %s", F_MINIMUM % inst->location().toRelativeString(ctx->documentPath()));
+		LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+			"$fs too small - clamping to %1$f",F_MINIMUM);
 		node->fs = F_MINIMUM;
 	}
 	if (node->fa < F_MINIMUM) {
-		PRINTB("WARNING: $fa too small - clamping to %f, %s", F_MINIMUM % inst->location().toRelativeString(ctx->documentPath()));
+		LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+			"$fa too small - clamping to %1$f",F_MINIMUM);
 		node->fa = F_MINIMUM;
 	}
 
 	switch (this->type)  {
 	case primitive_type_e::CUBE: {
-		auto size = c.lookup_variable("size");
-		auto center = c.lookup_variable("center");
-		if(size != ValuePtr::undefined){
+		const auto &size = c->lookup_variable("size");
+		const auto &center = c->lookup_variable("center");
+		if (size.isDefined()) {
 			bool converted=false;
-			converted |= size->getDouble(node->x);
-			converted |= size->getDouble(node->y);
-			converted |= size->getDouble(node->z);
-			converted |= size->getVec3(node->x, node->y, node->z);
-			if(!converted){
-				PRINTB("WARNING: Unable to convert cube(size=%s, ...) parameter to a number or a vec3 of numbers, %s", size->toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
-			}else if(OpenSCAD::rangeCheck){
+			converted |= size.getDouble(node->x);
+			converted |= size.getDouble(node->y);
+			converted |= size.getDouble(node->z);
+			converted |= size.getVec3(node->x, node->y, node->z);
+			if (!converted) {
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),"Unable to convert cube(size=%1$s, ...) parameter to a number or a vec3 of numbers",size.toEchoString());
+			} else if(OpenSCAD::rangeCheck) {
 				bool ok = (node->x > 0) && (node->y > 0) && (node->z > 0);
 				ok &= std::isfinite(node->x) && std::isfinite(node->y) && std::isfinite(node->z);
 				if(!ok){
-					PRINTB("WARNING: cube(size=%s, ...), %s",
-						size->toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
+					LOG(message_group::Warning,inst->location(),ctx->documentPath(),"cube(size=%1$s, ...)",size.toEchoString());
 				}
 			}
 		}
-		if (center->type() == Value::ValueType::BOOL) {
-			node->center = center->toBool();
+		if (center.type() == Value::Type::BOOL) {
+			node->center = center.toBool();
 		}
 		break;
 	}
 	case primitive_type_e::SPHERE: {
-		const auto r = lookup_radius(c, inst->location(), "d", "r");
-		if (r.type() == Value::ValueType::NUMBER) {
+		const auto r = lookup_radius(c.ctx, inst->location(), "d", "r");
+		if (r.type() == Value::Type::NUMBER) {
 			node->r1 = r.toDouble();
 			if (OpenSCAD::rangeCheck && (node->r1 <= 0 || !std::isfinite(node->r1))){
-				PRINTB("WARNING: sphere(r=%s), %s",
-					r.toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+					"sphere(r=%1$s)",r.toEchoString());
 			}
 		}
 		break;
 	}
 	case primitive_type_e::CYLINDER: {
-		const auto h = c.lookup_variable("h");
-		if (h->type() == Value::ValueType::NUMBER) {
-			node->h = h->toDouble();
+		const auto &h = c->lookup_variable("h");
+		if (h.type() == Value::Type::NUMBER) {
+			node->h = h.toDouble();
 		}
 
-		const auto r = lookup_radius(c, inst->location(), "d", "r");
-		const auto r1 = lookup_radius(c, inst->location(), "d1", "r1");
-		const auto r2 = lookup_radius(c, inst->location(), "d2", "r2");
-		if(r.type() == Value::ValueType::NUMBER && 
-			(r1.type() == Value::ValueType::NUMBER || r2.type() == Value::ValueType::NUMBER)
+		auto r = lookup_radius(c.ctx, inst->location(), "d", "r");
+		auto r1 = lookup_radius(c.ctx, inst->location(), "d1", "r1");
+		auto r2 = lookup_radius(c.ctx, inst->location(), "d2", "r2");
+		if(r.type() == Value::Type::NUMBER &&
+			(r1.type() == Value::Type::NUMBER || r2.type() == Value::Type::NUMBER)
 			){
-				PRINTB("WARNING: Cylinder parameters ambiguous, %s", inst->location().toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+					"Cylinder parameters ambiguous");
 		}
 
-		if (r.type() == Value::ValueType::NUMBER) {
+		if (r.type() == Value::Type::NUMBER) {
 			node->r1 = r.toDouble();
 			node->r2 = r.toDouble();
 		}
-		if (r1.type() == Value::ValueType::NUMBER) {
+		if (r1.type() == Value::Type::NUMBER) {
 			node->r1 = r1.toDouble();
 		}
-		if (r2.type() == Value::ValueType::NUMBER) {
+		if (r2.type() == Value::Type::NUMBER) {
 			node->r2 = r2.toDouble();
 		}
 
 		if(OpenSCAD::rangeCheck){
 			if (node->h <= 0 || !std::isfinite(node->h)){
-				PRINTB("WARNING: cylinder(h=%s, ...), %s",
-					h->toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),"cylinder(h=%1$s, ...)",h.toEchoString());
 			}
 			if (node->r1 < 0 || node->r2 < 0 || (node->r1 == 0 && node->r2 == 0) || !std::isfinite(node->r1) || !std::isfinite(node->r2)){
-				PRINTB("WARNING: cylinder(r1=%s, r2=%s, ...), %s",
-					(r1.type() == Value::ValueType::NUMBER ? r1.toEchoString() : r.toEchoString()) % 
-					(r2.type() == Value::ValueType::NUMBER ? r2.toEchoString() : r.toEchoString()) % 
-					inst->location().toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+					"cylinder(r1=%1$s, r2=%2$s, ...)",
+					(r1.type() == Value::Type::NUMBER ? r1.toEchoString() : r.toEchoString()),
+					(r2.type() == Value::Type::NUMBER ? r2.toEchoString() : r.toEchoString()));
 			}
 		}
 
-		auto center = c.lookup_variable("center");
-		if (center->type() == Value::ValueType::BOOL) {
-			node->center = center->toBool();
+		const auto &center = c->lookup_variable("center");
+		if (center.type() == Value::Type::BOOL) {
+			node->center = center.toBool();
 		}
 		break;
 	}
 	case primitive_type_e::POLYHEDRON: {
-		node->points = c.lookup_variable("points");
-		node->faces = c.lookup_variable("faces");
-		if (node->faces->type() == Value::ValueType::UNDEFINED) {
+		node->points = c->lookup_variable("points").clone();
+		node->faces = c->lookup_variable("faces").clone();
+		if (node->faces.type() == Value::Type::UNDEFINED) {
 			// backwards compatible
-			node->faces = c.lookup_variable("triangles", true);
-			if (node->faces->type() != Value::ValueType::UNDEFINED) {
-				printDeprecation("polyhedron(triangles=[]) will be removed in future releases. Use polyhedron(faces=[]) instead.");
+			node->faces = c->lookup_variable("triangles", true).clone();
+			if (node->faces.type() != Value::Type::UNDEFINED) {
+				LOG(message_group::Deprecated,Location::NONE,"","polyhedron(triangles=[]) will be removed in future releases. Use polyhedron(faces=[]) instead.");
 			}
 		}
 		break;
 	}
 	case primitive_type_e::SQUARE: {
-		auto size = c.lookup_variable("size");
-		auto center = c.lookup_variable("center");
-		if(size != ValuePtr::undefined){
+		const auto &size = c->lookup_variable("size");
+		const auto &center = c->lookup_variable("center");
+		if (size.isDefined()) {
 			bool converted=false;
-			converted |= size->getDouble(node->x);
-			converted |= size->getDouble(node->y);
-			converted |= size->getVec2(node->x, node->y);
-			if(!converted){
-				PRINTB("WARNING: Unable to convert square(size=%s, ...) parameter to a number or a vec2 of numbers, %s", size->toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
-			}else if(OpenSCAD::rangeCheck){
+			converted |= size.getDouble(node->x);
+			converted |= size.getDouble(node->y);
+			converted |= size.getVec2(node->x, node->y);
+			if (!converted) {
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),"Unable to convert square(size=%1$s, ...) parameter to a number or a vec2 of numbers",size.toEchoString());
+			} else if (OpenSCAD::rangeCheck) {
 				bool ok = true;
 				ok &= (node->x > 0) && (node->y > 0);
 				ok &= std::isfinite(node->x) && std::isfinite(node->y);
 				if(!ok){
-					PRINTB("WARNING: square(size=%s, ...), %s",
-						size->toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
+					LOG(message_group::Warning,inst->location(),ctx->documentPath(),"square(size=%1$s, ...)",size.toEchoString());
 				}
 			}
 		}
-		if (center->type() == Value::ValueType::BOOL) {
-			node->center = center->toBool();
+		if (center.type() == Value::Type::BOOL) {
+			node->center = center.toBool();
 		}
 		break;
 	}
 	case primitive_type_e::CIRCLE: {
-		const auto r = lookup_radius(c, inst->location(), "d", "r");
-		if (r.type() == Value::ValueType::NUMBER) {
+		const auto r = lookup_radius(c.ctx, inst->location(), "d", "r");
+		if (r.type() == Value::Type::NUMBER) {
 			node->r1 = r.toDouble();
 			if (OpenSCAD::rangeCheck && ((node->r1 <= 0) || !std::isfinite(node->r1))){
-				PRINTB("WARNING: circle(r=%s), %s",
-					r.toEchoString() % inst->location().toRelativeString(ctx->documentPath()));
+				LOG(message_group::Warning,inst->location(),ctx->documentPath(),
+					"circle(r=%1$s)",r.toEchoString());
 			}
 		}
 		break;
 	}
 	case primitive_type_e::POLYGON: {
-		node->points = c.lookup_variable("points");
-		node->paths = c.lookup_variable("paths");
+		node->points = c->lookup_variable("points").clone();
+		node->paths = c->lookup_variable("paths").clone();
 		break;
 	}
 	}
 
-	node->convexity = (int)c.lookup_variable("convexity", true)->toDouble();
+	node->convexity = (int)c->lookup_variable("convexity", true).toDouble();
 	if (node->convexity < 1) node->convexity = 1;
 
 	return node;
@@ -345,7 +334,7 @@ struct point2d {
 
 static void generate_circle(point2d *circle, double r, int fragments)
 {
-	for (int i=0; i<fragments; i++) {
+	for (int i=0; i<fragments; ++i) {
 		double phi = (360.0 * i) / fragments;
 		circle[i].x = r * cos_degrees(phi);
 		circle[i].y = r * sin_degrees(phi);
@@ -436,7 +425,7 @@ const Geometry *PrimitiveNode::createGeometry() const
 			auto ring = std::vector<ring_s>(rings);
 
 //		double offset = 0.5 * ((fragments / 2) % 2);
-			for (int i = 0; i < rings; i++) {
+			for (int i = 0; i < rings; ++i) {
 //			double phi = (180.0 * (i + offset)) / (fragments/2);
 				double phi = (180.0 * (i + 0.5)) / rings;
 				double r = r1 * sin_degrees(phi);
@@ -446,10 +435,10 @@ const Geometry *PrimitiveNode::createGeometry() const
 			}
 
 			p->append_poly();
-			for (int i = 0; i < fragments; i++)
+			for (int i = 0; i < fragments; ++i)
 				p->append_vertex(ring[0].points[i].x, ring[0].points[i].y, ring[0].z);
 
-			for (int i = 0; i < rings-1; i++) {
+			for (int i = 0; i < rings-1; ++i) {
 				auto r1 = &ring[i];
 				auto  r2 = &ring[i+1];
 				int r1i = 0, r2i = 0;
@@ -477,9 +466,9 @@ const Geometry *PrimitiveNode::createGeometry() const
 			}
 
 			p->append_poly();
-			for (int i = 0; i < fragments; i++) {
-				p->insert_vertex(ring[rings-1].points[i].x, 
-												 ring[rings-1].points[i].y, 
+			for (int i = 0; i < fragments; ++i) {
+				p->insert_vertex(ring[rings-1].points[i].x,
+												 ring[rings-1].points[i].y,
 												 ring[rings-1].z);
 			}
 		}
@@ -507,8 +496,8 @@ const Geometry *PrimitiveNode::createGeometry() const
 
 			generate_circle(circle1.data(), r1, fragments);
 			generate_circle(circle2.data(), r2, fragments);
-		
-			for (int i=0; i<fragments; i++) {
+
+			for (int i=0; i<fragments; ++i) {
 				int j = (i+1) % fragments;
 				if (r1 == r2) {
 					p->append_poly();
@@ -534,13 +523,13 @@ const Geometry *PrimitiveNode::createGeometry() const
 
 			if (this->r1 > 0) {
 				p->append_poly();
-				for (int i=0; i<fragments; i++)
+				for (int i=0; i<fragments; ++i)
 					p->insert_vertex(circle1[i].x, circle1[i].y, z1);
 			}
 
 			if (this->r2 > 0) {
 				p->append_poly();
-				for (int i=0; i<fragments; i++)
+				for (int i=0; i<fragments; ++i)
 					p->append_vertex(circle2[i].x, circle2[i].y, z2);
 			}
 		}
@@ -550,21 +539,27 @@ const Geometry *PrimitiveNode::createGeometry() const
 		auto p = new PolySet(3);
 		g = p;
 		p->setConvexity(this->convexity);
-		for (size_t i=0; i<this->faces->toVector().size(); i++)	{
+		const auto &pts = this->points.toVector();
+		size_t face_i = 0;
+		for (const auto &face : this->faces.toVector())	{
 			p->append_poly();
-			const auto &vec = this->faces->toVector()[i]->toVector();
-			for (size_t j=0; j<vec.size(); j++) {
-				size_t pt = (size_t)vec[j]->toDouble();
-				if (pt < this->points->toVector().size()) {
+			size_t fp_i = 0;
+			for (const auto &pt_i_val : face.toVector()) {
+				size_t pt_i = (size_t)pt_i_val.toDouble();
+				if (pt_i < pts.size()) {
 					double px, py, pz;
-					if (!this->points->toVector()[pt]->getVec3(px, py, pz, 0.0) ||
+					if (!pts[pt_i].getVec3(px, py, pz, 0.0) ||
 					    !std::isfinite(px) || !std::isfinite(py) || !std::isfinite(pz)) {
-						PRINTB("ERROR: Unable to convert point at index %d to a vec3 of numbers, %s", j % this->modinst->location().toRelativeString(this->document_path));
+						LOG(message_group::Error,this->modinst->location(),this->document_path,"Unable to convert points[%1$d] = %2$s to a vec3 of numbers",pt_i,pts[pt_i].toEchoString());
 						return p;
 					}
 					p->insert_vertex(px, py, pz);
+				} else {
+					LOG(message_group::Warning,this->modinst->location(),this->document_path,"Point index %1$d is out of bounds (from faces[%2$d][%3$d])",pt_i,face_i,fp_i);
 				}
+				++fp_i;
 			}
+			++face_i;
 		}
 	}
 		break;
@@ -595,7 +590,7 @@ const Geometry *PrimitiveNode::createGeometry() const
 
 			Outline2d o;
 			o.vertices.resize(fragments);
-			for (int i=0; i < fragments; i++) {
+			for (int i=0; i < fragments; ++i) {
 				double phi = (360.0 * i) / fragments;
 				o.vertices[i] = {this->r1 * cos_degrees(phi), this->r1 * sin_degrees(phi)};
 			}
@@ -607,41 +602,44 @@ const Geometry *PrimitiveNode::createGeometry() const
 	case primitive_type_e::POLYGON:	{
 			auto p = new Polygon2d();
 			g = p;
-
 			Outline2d outline;
 			double x,y;
-			const auto &vec = this->points->toVector();
-			for (unsigned int i=0;i<vec.size();i++) {
-				const auto &val = *vec[i];
+			size_t i = 0;
+			for (const auto &val : this->points.toVector()) {
 				if (!val.getVec2(x, y) || std::isinf(x) || std::isinf(y)) {
-					PRINTB("ERROR: Unable to convert point %s at index %d to a vec2 of numbers, %s", 
-								 val.toEchoString() % i % this->modinst->location().toRelativeString(this->document_path));
+					LOG(message_group::Error,this->modinst->location(),this->document_path,"Unable to convert points[%1$d] = %2$s to a vec2 of numbers",i,val.toEchoString());
 					return p;
 				}
 				outline.vertices.emplace_back(x, y);
+				++i;
 			}
 
-			if (this->paths->toVector().size() == 0 && outline.vertices.size() > 2) {
+			if (this->paths.toVector().size() == 0 && outline.vertices.size() > 2) {
 				p->addOutline(outline);
 			}
 			else {
-				for (const auto &polygon : this->paths->toVector()) {
+				size_t path_i = 0;
+				for (const auto &polygon : this->paths.toVector()) {
 					Outline2d curroutline;
-					for (const auto &index : polygon->toVector()) {
-						unsigned int idx = (unsigned int)index->toDouble();
+					size_t path_pt_i = 0;
+					for (const auto &index : polygon.toVector()) {
+						unsigned int idx = (unsigned int)index.toDouble();
 						if (idx < outline.vertices.size()) {
 							curroutline.vertices.push_back(outline.vertices[idx]);
+						} else {
+							LOG(message_group::Warning,this->modinst->location(),this->document_path,"Point index %1$d is out of bounds (from paths[%2$d][%3$d])",idx , path_i , path_pt_i);
 						}
-						// FIXME: Warning on out of bounds?
+						++path_pt_i;
 					}
 					p->addOutline(curroutline);
+					++path_i;
 				}
 			}
-        
+
 			if (p->outlines().size() > 0) {
 				p->setConvexity(convexity);
 			}
-	}
+		}
 	}
 
 	return g;
@@ -669,9 +667,9 @@ std::string PrimitiveNode::toString() const
 			break;
 	case primitive_type_e::POLYHEDRON:
 		stream << "(points = ";
-		this->points->toStream(stream);
+		this->points.toStream(stream);
 		stream << ", faces = ";
-		this->faces->toStream(stream);
+		this->faces.toStream(stream);
 		stream << ", convexity = " << this->convexity << ")";
 			break;
 	case primitive_type_e::SQUARE:
@@ -684,9 +682,9 @@ std::string PrimitiveNode::toString() const
 		break;
 	case primitive_type_e::POLYGON:
 		stream << "(points = ";
-		this->points->toStream(stream);
+		this->points.toStream(stream);
 		stream << ", paths = ";
-		this->paths->toStream(stream);
+		this->paths.toStream(stream);
 		stream << ", convexity = " << this->convexity << ")";
 			break;
 	default:
@@ -698,11 +696,50 @@ std::string PrimitiveNode::toString() const
 
 void register_builtin_primitives()
 {
-	Builtins::init("cube", new PrimitiveModule(primitive_type_e::CUBE));
-	Builtins::init("sphere", new PrimitiveModule(primitive_type_e::SPHERE));
-	Builtins::init("cylinder", new PrimitiveModule(primitive_type_e::CYLINDER));
-	Builtins::init("polyhedron", new PrimitiveModule(primitive_type_e::POLYHEDRON));
-	Builtins::init("square", new PrimitiveModule(primitive_type_e::SQUARE));
-	Builtins::init("circle", new PrimitiveModule(primitive_type_e::CIRCLE));
-	Builtins::init("polygon", new PrimitiveModule(primitive_type_e::POLYGON));
+	Builtins::init("cube" , new PrimitiveModule(primitive_type_e::CUBE),
+				{
+					"cube(size)",
+					"cube([width, depth, height])",
+					"cube([width, depth, height], center = true)",
+				});
+
+	Builtins::init("sphere", new PrimitiveModule(primitive_type_e::SPHERE),
+				{
+					"sphere(radius)",
+					"sphere(r = radius)",
+					"sphere(d = diameter)",
+				});
+
+	Builtins::init("cylinder", new PrimitiveModule(primitive_type_e::CYLINDER),
+				{
+					"cylinder(h, r1, r2)",
+					"cylinder(h = height, r = radius, center = true)",
+					"cylinder(h = height, r1 = bottom, r2 = top, center = true)",
+					"cylinder(h = height, d = diameter, center = true)",
+					"cylinder(h = height, d1 = bottom, d2 = top, center = true)",
+				});
+
+	Builtins::init("polyhedron", new PrimitiveModule(primitive_type_e::POLYHEDRON),
+				{
+					"polyhedron(points, faces, convexity)",
+				});
+
+	Builtins::init("square", new PrimitiveModule(primitive_type_e::SQUARE),
+				{
+					"square(size, center = true)",
+					"square([width,height], center = true)",
+				});
+
+	Builtins::init("circle", new PrimitiveModule(primitive_type_e::CIRCLE),
+				{
+					"circle(radius)",
+					"circle(r = radius)",
+					"circle(d = diameter)",
+				});
+
+	Builtins::init("polygon", new PrimitiveModule(primitive_type_e::POLYGON),
+				{
+					"polygon([points])",
+					"polygon([points], [paths])",
+				});
 }
