@@ -2,6 +2,7 @@
 #include "node.h"
 
 int progress_report_count;
+int _progress_mark;
 void (*progress_report_f)(const class AbstractNode*, void*, int);
 void *progress_report_userdata;
 
@@ -22,7 +23,14 @@ void progress_report_fin()
 
 void progress_update(const AbstractNode *node, int mark)
 {
-	if (progress_report_f)
-		progress_report_f(node, progress_report_userdata, mark);
+	if (progress_report_f) {
+		_progress_mark = mark;
+		progress_report_f(node, progress_report_userdata, _progress_mark);
+	}
 }
 
+void progress_tick()
+{
+	if (progress_report_f)
+		progress_report_f(nullptr, progress_report_userdata, ++_progress_mark);
+}
