@@ -7,7 +7,7 @@ class ProjectionNode : public AbstractPolyNode
 {
 public:
 	VISITABLE();
-	ProjectionNode(const ModuleInstantiation *mi) : AbstractPolyNode(mi), convexity(1), cut_mode(false) { }
+	ProjectionNode(const ModuleInstantiation *mi, const std::shared_ptr<EvalContext> &ctx) : AbstractPolyNode(mi, ctx), convexity(1), cut_mode(false) { }
 	std::string toString() const override;
 	std::string name() const override { return "projection"; }
 
