@@ -8,18 +8,19 @@
 // is already defined, so the workaround defines sprintf as itself.
 #ifdef __MINGW32__
 #define _GL_STDIO_H
+#undef sprintf
 #define sprintf sprintf
 #endif
 
 #include <QString>
 #include "printutils.h"
 
-#define N_(String) String
-
-inline QString _( const char *msgid, int category )
+inline QString q_(const char *msgid, const char *msgctxt)
 {
-	Q_UNUSED( category );
-	return QString::fromUtf8( _( msgid ) );
+	return QString::fromUtf8(msgctxt ?
+		_(msgid, msgctxt):
+		_(msgid)
+		);
 }
 
 #endif
