@@ -1,10 +1,12 @@
 #pragma once
 
+#include "GLView.h"
 #include "system-gl.h"
 #include "linalg.h"
 #include "memory.h"
 #include "colormap.h"
 #include "enums.h"
+#include "Geometry.h"
 
 #ifdef _MSC_VER // NULL
 #include <cstdlib>
@@ -16,8 +18,9 @@ public:
 	Renderer();
 	virtual ~Renderer() {}
 	virtual void draw(bool showfaces, bool showedges) const = 0;
+	virtual void draw_with_shader(const GLView::shaderinfo_t *) const  { this->draw(true, true); }
 	virtual BoundingBox getBoundingBox() const = 0;
-	
+
 #define CSGMODE_DIFFERENCE_FLAG 0x10
 	enum csgmode_e {
 		CSGMODE_NONE                  = 0x00,
@@ -39,18 +42,20 @@ public:
 		CUTOUT_EDGES,
 		HIGHLIGHT_EDGES,
 		BACKGROUND_EDGES,
+		CGAL_FACE_2D_COLOR,
+		CGAL_EDGE_2D_COLOR,
 		EMPTY_SPACE
 	};
 
 	virtual bool getColor(ColorMode colormode, Color4f &col) const;
-	virtual void setColor(const float color[4], GLint *shaderinfo = nullptr) const;
-	virtual void setColor(ColorMode colormode, GLint *shaderinfo = nullptr) const;
-	virtual Color4f setColor(ColorMode colormode, const float color[4], GLint *shaderinfo = nullptr) const;
+	virtual void setColor(const float color[4], const GLView::shaderinfo_t *shaderinfo = nullptr) const;
+	virtual void setColor(ColorMode colormode, const GLView::shaderinfo_t *shaderinfo = nullptr) const;
+	virtual Color4f setColor(ColorMode colormode, const float color[4], const GLView::shaderinfo_t *shaderinfo = nullptr) const;
 	virtual void setColorScheme(const ColorScheme &cs);
 
-	static csgmode_e get_csgmode(const bool highlight_mode, const bool background_mode, const OpenSCADOperator type=OpenSCADOperator::UNION);
-	static void render_surface(shared_ptr<const class Geometry> geom, csgmode_e csgmode, const Transform3d &m, GLint *shaderinfo = nullptr);
-	static void render_edges(shared_ptr<const Geometry> geom, csgmode_e csgmode);
+	virtual csgmode_e get_csgmode(const bool highlight_mode, const bool background_mode, const OpenSCADOperator type=OpenSCADOperator::UNION) const;
+	virtual void render_surface(shared_ptr<const class Geometry> geom, csgmode_e csgmode, const Transform3d &m, const GLView::shaderinfo_t *shaderinfo = nullptr) const;
+	virtual void render_edges(shared_ptr<const Geometry> geom, csgmode_e csgmode) const;
 
 protected:
 	std::map<ColorMode,Color4f> colormap;
