@@ -7,7 +7,7 @@ class RenderNode : public AbstractNode
 {
 public:
 	VISITABLE();
-	RenderNode(const ModuleInstantiation *mi) : AbstractNode(mi), convexity(1) { }
+	RenderNode(const ModuleInstantiation *mi, const std::shared_ptr<EvalContext> &ctx) : AbstractNode(mi, ctx), convexity(1) { }
 	std::string toString() const override;
 	std::string name() const override { return "render"; }
 
