@@ -33,7 +33,7 @@
 #include "builtin.h"
 #include "polyset.h"
 #include "handle_dep.h"
-
+#include "boost-utils.h"
 #include <sstream>
 #include <boost/assign/std/vector.hpp>
 using namespace boost::assign; // bring 'operator+=()' into scope
@@ -45,49 +45,48 @@ class RotateExtrudeModule : public AbstractModule
 {
 public:
 	RotateExtrudeModule() { }
-	AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const override;
+	AbstractNode *instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const override;
 };
 
-AbstractNode *RotateExtrudeModule::instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const
+AbstractNode *RotateExtrudeModule::instantiate(const std::shared_ptr<Context>& ctx, const ModuleInstantiation *inst, const std::shared_ptr<EvalContext>& evalctx) const
 {
-	auto node = new RotateExtrudeNode(inst);
+	auto node = new RotateExtrudeNode(inst, evalctx);
 
-	AssignmentList args{Assignment("file"), Assignment("layer"), Assignment("origin"), Assignment("scale")};
-	AssignmentList optargs{Assignment("convexity"), Assignment("angle")};
+	AssignmentList args{assignment("file"), assignment("layer"), assignment("origin"), assignment("scale")};
+	AssignmentList optargs{assignment("convexity"), assignment("angle")};
 
-	Context c(ctx);
-	c.setVariables(evalctx, args, optargs);
-	inst->scope.apply(*evalctx);
+	ContextHandle<Context> c{Context::create<Context>(ctx)};
+	c->setVariables(evalctx, args, optargs);
+	inst->scope.apply(evalctx);
 
-	node->fn = c.lookup_variable("$fn")->toDouble();
-	node->fs = c.lookup_variable("$fs")->toDouble();
-	node->fa = c.lookup_variable("$fa")->toDouble();
-    
+	node->fn = c->lookup_variable("$fn").toDouble();
+	node->fs = c->lookup_variable("$fs").toDouble();
+	node->fa = c->lookup_variable("$fa").toDouble();
 
-	auto file = c.lookup_variable("file");
-	auto layer = c.lookup_variable("layer", true);
-	auto convexity = c.lookup_variable("convexity", true);
-	auto origin = c.lookup_variable("origin", true);
-	auto scale = c.lookup_variable("scale", true);
-	auto angle = c.lookup_variable("angle", true);
-    
-	if (!file->isUndefined()) {
-		printDeprecation("Support for reading files in rotate_extrude will be removed in future releases. Use a child import() instead.");
-		auto filename = lookup_file(file->toString(), inst->path(), c.documentPath());
+	const auto &file = c->lookup_variable("file");
+	const auto &layer = c->lookup_variable("layer", true);
+	const auto &convexity = c->lookup_variable("convexity", true);
+	const auto &origin = c->lookup_variable("origin", true);
+	const auto &scale = c->lookup_variable("scale", true);
+	const auto &angle = c->lookup_variable("angle", true);
+
+	if (!file.isUndefined()) {
+		LOG(message_group::Deprecated,Location::NONE,"","Support for reading files in rotate_extrude will be removed in future releases. Use a child import() instead.");
+		auto filename = lookup_file(file.toString(), inst->path(), c->documentPath());
 		node->filename = filename;
 		handle_dep(filename);
 	}
 
-	node->layername = layer->isUndefined() ? "" : layer->toString();
-	node->convexity = static_cast<int>(convexity->toDouble());
-	bool originOk = origin->getVec2(node->origin_x, node->origin_y);
+	node->layername = layer.isUndefined() ? "" : layer.toString();
+	node->convexity = static_cast<int>(convexity.toDouble());
+	bool originOk = origin.getVec2(node->origin_x, node->origin_y);
 	originOk &= std::isfinite(node->origin_x) && std::isfinite(node->origin_y);
-	if(origin!=ValuePtr::undefined && !originOk){
-		PRINTB("WARNING: rotate_extrude(..., origin=%s) could not be converted, %s", origin->toEchoString() % evalctx->loc.toRelativeString(ctx->documentPath()));
+	if(origin.isDefined() && !originOk){
+		LOG(message_group::Warning,evalctx->loc,ctx->documentPath(),"rotate_extrude(..., origin=%1$s) could not be converted",origin.toEchoString());
 	}
-	node->scale = scale->toDouble();
+	node->scale = scale.toDouble();
 	node->angle = 360;
-	angle->getFiniteDouble(node->angle);
+	angle.getFiniteDouble(node->angle);
 
 	if (node->convexity <= 0)
 		node->convexity = 2;
@@ -111,7 +110,7 @@ std::string RotateExtrudeNode::toString() const
 	std::ostringstream stream;
 
 	stream << this->name() << "(";
-	if (!this->filename.empty()) { // Ignore deprecated parameters if empty 
+	if (!this->filename.empty()) { // Ignore deprecated parameters if empty
 		fs::path path((std::string)this->filename);
 		stream <<
 			"file = " << this->filename << ", "
@@ -132,5 +131,9 @@ std::string RotateExtrudeNode::toString() const
 void register_builtin_dxf_rotate_extrude()
 {
 	Builtins::init("dxf_rotate_extrude", new RotateExtrudeModule());
-	Builtins::init("rotate_extrude", new RotateExtrudeModule());
+
+	Builtins::init("rotate_extrude", new RotateExtrudeModule(),
+				{
+					"rotate_extrude(angle = 360, convexity = 2)",
+				});
 }
