@@ -1,25 +1,37 @@
+#include <ciso646> // C alternative tokens (xor)
 #include <stdlib.h>
 #include <algorithm>
+#include <boost/filesystem.hpp>
+#include <boost/property_tree/ptree.hpp>
+#include <boost/property_tree/json_parser.hpp>
+#include "boost-utils.h"
 #include <QString>
 #include <QChar>
-#include "scintillaeditor.h"
+#include <QShortcut>
 #include <Qsci/qscicommandset.h>
+
+#include "scintillaeditor.h"
 #include "Preferences.h"
 #include "PlatformUtils.h"
 #include "settings.h"
-#include <ciso646> // C alternative tokens (xor)
-#include <boost/filesystem.hpp>
+#include "QSettingsCached.h"
+
+#include <QWheelEvent>
+#include<QPoint>
+
 namespace fs=boost::filesystem;
 
+const QString ScintillaEditor::cursorPlaceHolder = "^~^";
+
 class SettingsConverter {
 public:
-	QsciScintilla::WrapMode toWrapMode(Value val);
-	QsciScintilla::WrapVisualFlag toLineWrapVisualization(Value val);
-	QsciScintilla::WrapIndentMode toLineWrapIndentationStyle(Value val);
-	QsciScintilla::WhitespaceVisibility toShowWhitespaces(Value val);
+	QsciScintilla::WrapMode toWrapMode(const Value &val);
+	QsciScintilla::WrapVisualFlag toLineWrapVisualization(const Value &val);
+	QsciScintilla::WrapIndentMode toLineWrapIndentationStyle(const Value &val);
+	QsciScintilla::WhitespaceVisibility toShowWhitespaces(const Value &val);
 };
 
-QsciScintilla::WrapMode SettingsConverter::toWrapMode(Value val)
+QsciScintilla::WrapMode SettingsConverter::toWrapMode(const Value &val)
 {
 	auto strVal = val.toString();
 	if (strVal == "Char") {
@@ -31,7 +43,7 @@ QsciScintilla::WrapMode SettingsConverter::toWrapMode(Value val)
 	}
 }
 
-QsciScintilla::WrapVisualFlag SettingsConverter::toLineWrapVisualization(Value val)
+QsciScintilla::WrapVisualFlag SettingsConverter::toLineWrapVisualization(const Value &val)
 {
 	auto strVal = val.toString();
 	if (strVal == "Text") {
@@ -47,7 +59,7 @@ QsciScintilla::WrapVisualFlag SettingsConverter::toLineWrapVisualization(Value v
 	}
 }
 
-QsciScintilla::WrapIndentMode SettingsConverter::toLineWrapIndentationStyle(Value val)
+QsciScintilla::WrapIndentMode SettingsConverter::toLineWrapIndentationStyle(const Value &val)
 {
 	auto strVal = val.toString();
 	if (strVal == "Same") {
@@ -59,7 +71,7 @@ QsciScintilla::WrapIndentMode SettingsConverter::toLineWrapIndentationStyle(Valu
 	}
 }
 
-QsciScintilla::WhitespaceVisibility SettingsConverter::toShowWhitespaces(Value val)
+QsciScintilla::WhitespaceVisibility SettingsConverter::toShowWhitespaces(const Value &val)
 {
 	auto strVal = val.toString();
 	if (strVal == "Always") {
@@ -78,7 +90,7 @@ EditorColorScheme::EditorColorScheme(fs::path path) : path(path)
 		_name = QString::fromStdString(pt.get<std::string>("name"));
 		_index = pt.get<int>("index");
 	} catch (const std::exception & e) {
-		PRINTB("Error reading color scheme file '%s': %s", path.generic_string() % e.what());
+		LOG(message_group::None,Location::NONE,"","Error reading color scheme file '%1$s': %2$s",path.generic_string(),e.what());
 		_name = "";
 		_index = 0;
 	}
@@ -111,9 +123,15 @@ const boost::property_tree::ptree & EditorColorScheme::propertyTree() const
 
 ScintillaEditor::ScintillaEditor(QWidget *parent) : EditorInterface(parent)
 {
+	api = nullptr;
+	lexer = nullptr;
 	scintillaLayout = new QVBoxLayout(this);
 	qsci = new QsciScintilla(this);
 
+	contentsRendered = false;
+	findState = 0; //FIND_HIDDEN
+	filepath = "";
+
 	// Force EOL mode to Unix, since QTextStream will manage local EOL modes.
 	qsci->setEolMode(QsciScintilla::EolUnix);
 
@@ -138,22 +156,114 @@ ScintillaEditor::ScintillaEditor(QWidget *parent) : EditorInterface(parent)
 	c->setKey(Qt::Key_Z | Qt::CTRL | Qt::SHIFT);
 	c->setAlternateKey(Qt::Key_Y | Qt::CTRL);
 
+#ifdef Q_OS_MAC
+	const unsigned long modifier = Qt::META;
+#else
+	const unsigned long modifier = Qt::CTRL;
+#endif
+
+	QShortcut *shortcutCalltip;
+	shortcutCalltip = new QShortcut(modifier | Qt::SHIFT | Qt::Key_Space, this);
+	connect(shortcutCalltip, &QShortcut::activated, [=]() { qsci->callTip(); });
+
+	QShortcut *shortcutAutocomplete;
+	shortcutAutocomplete = new QShortcut(modifier | Qt::Key_Space, this);
+	connect(shortcutAutocomplete, &QShortcut::activated, [=]() { qsci->autoCompleteFromAll(); });
+
 	scintillaLayout->setContentsMargins(0, 0, 0, 0);
 	scintillaLayout->addWidget(qsci);
 
-	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator, errorIndicatorNumber);
-	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator , findIndicatorNumber); 
-	qsci->markerDefine(QsciScintilla::Circle, markerNumber);
 	qsci->setUtf8(true);
 	qsci->setFolding(QsciScintilla::BoxedTreeFoldStyle, 4);
+	qsci->setCaretLineVisible(true);
 
-	this->lexer = new ScadLexer(this);
-	qsci->setLexer(this->lexer);
+	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator, errorIndicatorNumber);
+	qsci->indicatorDefine(QsciScintilla::RoundBoxIndicator , findIndicatorNumber);
+	qsci->markerDefine(QsciScintilla::Circle, errMarkerNumber);
+	qsci->markerDefine(QsciScintilla::Bookmark, bmMarkerNumber);
+
+	qsci->setMarginType(numberMargin, QsciScintilla::NumberMargin);
+	qsci->setMarginLineNumbers(numberMargin, true);
+	qsci->setMarginMarkerMask(numberMargin, 0);
+
+	qsci->setMarginType(symbolMargin, QsciScintilla::SymbolMargin);
+	qsci->setMarginLineNumbers(symbolMargin, false);
+	qsci->setMarginWidth(symbolMargin, 0);
+	qsci->setMarginMarkerMask(symbolMargin, 1 << errMarkerNumber | 1 << bmMarkerNumber);
+
+	setLexer(new ScadLexer(this));
 	initMargin();
 
 	connect(qsci, SIGNAL(textChanged()), this, SIGNAL(contentsChanged()));
-	connect(qsci, SIGNAL(modificationChanged(bool)), this, SIGNAL(modificationChanged(bool)));
+	connect(qsci, SIGNAL(modificationChanged(bool)), this, SLOT(fireModificationChanged(bool)));
+	connect(qsci, SIGNAL(userListActivated(int, const QString &)), this, SLOT(onUserListSelected(const int, const QString &)));
 	qsci->installEventFilter(this);
+	qsci->viewport()->installEventFilter(this);
+
+    qsci->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(qsci, SIGNAL(customContextMenuRequested(const QPoint &)), this, SIGNAL(showContextMenuEvent(const QPoint &)));
+
+	qsci->indicatorDefine(QsciScintilla::ThinCompositionIndicator, hyperlinkIndicatorNumber);
+	qsci->setIndicatorHoverStyle(QsciScintilla::DotBoxIndicator, hyperlinkIndicatorNumber);
+	connect(qsci, SIGNAL(indicatorClicked(int, int, Qt::KeyboardModifiers)), this, SLOT(onIndicatorClicked(int, int, Qt::KeyboardModifiers)));
+
+#if QSCINTILLA_VERSION >= 0x020b00
+	connect(qsci, SIGNAL(SCN_URIDROPPED(const QUrl&)), this, SIGNAL(uriDropped(const QUrl&)));
+#endif
+
+    // Disabling buffered drawing resolves non-integer HiDPI scaling.
+    qsci->SendScintilla(QsciScintillaBase::SCI_SETBUFFEREDDRAW, false);
+}
+
+QPoint ScintillaEditor::mapToGlobal(const QPoint &pos)
+{
+	return qsci->mapToGlobal(pos);
+}
+
+QMenu * ScintillaEditor::createStandardContextMenu()
+{
+	return qsci->createStandardContextMenu();
+}
+
+void ScintillaEditor::addTemplate()
+{
+	addTemplate(PlatformUtils::resourceBasePath());
+	addTemplate(PlatformUtils::userConfigPath());
+	for(auto key: templateMap.keys())
+	{
+		userList.append(key);
+	}
+}
+
+void ScintillaEditor::addTemplate(const fs::path path)
+{
+	const auto template_path = path / "templates";
+
+	if (fs::exists(template_path) && fs::is_directory(template_path)) {
+		for (const auto& dirEntry : boost::make_iterator_range(fs::directory_iterator{template_path}, {})) {
+			if (!fs::is_regular_file(dirEntry.status())) continue;
+
+			const auto &path = dirEntry.path();
+			if (!(path.extension() == ".json")) continue;
+
+			boost::property_tree::ptree pt;
+			try {
+				boost::property_tree::read_json(path.generic_string().c_str(), pt);
+				const QString key = QString::fromStdString(pt.get<std::string>("key"));
+				const QString content = QString::fromStdString(pt.get<std::string>("content"));
+				const int cursor_offset = pt.get<int>("offset", -1);
+
+				templateMap.insert(key, ScadTemplate(content, cursor_offset));
+			} catch (const std::exception & e) {
+				LOG(message_group::None,Location::NONE,"","Error reading template file '%1$s': %2$s",path.generic_string(),e.what());
+			}
+		}
+	}
+}
+
+void ScintillaEditor::displayTemplates()
+{
+	qsci->showUserList(1, userList);
 }
 
 /**
@@ -184,11 +294,48 @@ void ScintillaEditor::applySettings()
 
 	qsci->setBraceMatching(s->get(Settings::Settings::enableBraceMatching).toBool() ? QsciScintilla::SloppyBraceMatch : QsciScintilla::NoBraceMatch);
 	qsci->setCaretLineVisible(s->get(Settings::Settings::highlightCurrentLine).toBool());
-    auto value = s->get(Settings::Settings::enableLineNumbers).toBool();
-    qsci->setMarginLineNumbers(1,value);
+	onTextChanged();
 
-    if (!value) qsci->setMarginWidth(1,20);
-    else qsci->setMarginWidth(1,QString(trunc(log10(qsci->lines())+4), '0'));
+	setupAutoComplete(false);
+}
+
+void ScintillaEditor::setupAutoComplete(const bool forceOff)
+{
+	if (qsci->isListActive()) {
+		qsci->cancelList();
+	}
+
+	if (qsci->isCallTipActive()) {
+		qsci->SendScintilla(QsciScintilla::SCI_CALLTIPCANCEL);
+	}
+
+	const bool configValue = Preferences::inst()->getValue("editor/enableAutocomplete").toBool();
+	const bool enable = configValue && !forceOff;
+
+	if (enable) {
+		qsci->setAutoCompletionSource(QsciScintilla::AcsAPIs);
+		qsci->setAutoCompletionFillupsEnabled(false);
+		qsci->setAutoCompletionFillups("(");
+		qsci->setCallTipsVisible(10);
+		qsci->setCallTipsStyle(QsciScintilla::CallTipsContext);
+	} else {
+		qsci->setAutoCompletionSource(QsciScintilla::AcsNone);
+		qsci->setAutoCompletionFillupsEnabled(false);
+		qsci->setCallTipsStyle(QsciScintilla::CallTipsNone);
+	}
+
+	int val = Preferences::inst()->getValue("editor/characterThreshold").toInt();
+	qsci->setAutoCompletionThreshold(val <= 0 ? 1 : val);
+}
+
+void ScintillaEditor::fireModificationChanged(bool b)
+{
+	emit modificationChanged(b, this);
+}
+
+void ScintillaEditor::public_applySettings()
+{
+	applySettings();
 }
 
 void ScintillaEditor::setPlainText(const QString &text)
@@ -205,9 +352,7 @@ QString ScintillaEditor::toPlainText()
 void ScintillaEditor::setContentModified(bool modified)
 {
 	// FIXME: Due to an issue with QScintilla, we need to do this on the document itself.
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	qsci->SCN_SAVEPOINTLEFT();
-#endif
 	qsci->setModified(modified);
 }
 
@@ -221,7 +366,8 @@ void ScintillaEditor::highlightError(int error_pos)
 	int line, index;
 	qsci->lineIndexFromPosition(error_pos, &line, &index);
 	qsci->fillIndicatorRange(line, index, line, index + 1, errorIndicatorNumber);
-	qsci->markerAdd(line, markerNumber);
+	qsci->markerAdd(line, errMarkerNumber);
+	updateSymbolMarginVisibility();
 }
 
 void ScintillaEditor::unhighlightLastError()
@@ -230,22 +376,14 @@ void ScintillaEditor::unhighlightLastError()
 	int line, index;
 	qsci->lineIndexFromPosition(totalLength, &line, &index);
 	qsci->clearIndicatorRange(0, 0, line, index, errorIndicatorNumber);
-	qsci->markerDeleteAll(markerNumber);
+	qsci->markerDeleteAll(errMarkerNumber);
+	updateSymbolMarginVisibility();
 }
 
 QColor ScintillaEditor::readColor(const boost::property_tree::ptree &pt, const std::string name, const QColor defaultColor)
 {
 	try {
 		const auto val = pt.get<std::string>(name);
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-		if ((val.length() == 9) && (val.at(0) == '#')) {
-			const auto rgb = std::string("#") + val.substr(3);
-			QColor qcol(rgb.c_str());
-			auto alpha = std::strtoul(val.substr(1, 2).c_str(), 0, 16);
-			qcol.setAlpha(alpha);
-			return qcol;
-		}
-#endif
 		return QColor(val.c_str());
 	} catch (const std::exception &e) {
 		return defaultColor;
@@ -272,6 +410,15 @@ int ScintillaEditor::readInt(const boost::property_tree::ptree &pt, const std::s
 	}
 }
 
+void ScintillaEditor::setLexer(ScadLexer *newLexer)
+{
+	delete this->api;
+	this->qsci->setLexer(newLexer);
+	this->api = new ScadApi(this->qsci, newLexer);
+	delete this->lexer;
+	this->lexer = newLexer;
+}
+
 void ScintillaEditor::setColormap(const EditorColorScheme *colorScheme)
 {
 	const auto & pt = colorScheme->propertyTree();
@@ -293,9 +440,7 @@ void ScintillaEditor::setColormap(const EditorColorScheme *colorScheme)
 			newLexer->setKeywords(4, readString(keywords.get(), "keyword-set3", ""));
 		}
 
-		qsci->setLexer(newLexer);
-		delete this->lexer;
-		this->lexer = newLexer;
+		setLexer(newLexer);
 
 		// All other properties must be set after attaching to QSCintilla so
 		// the editor gets the change events and updates itself to match
@@ -324,11 +469,15 @@ void ScintillaEditor::setColormap(const EditorColorScheme *colorScheme)
 		qsci->setCaretForegroundColor(readColor(caret, "foreground", textColor));
 		qsci->setCaretLineBackgroundColor(readColor(caret, "line-background", paperColor));
 
-		qsci->setMarkerBackgroundColor(readColor(colors, "error-marker", QColor(255, 0, 0, 100)), markerNumber);
+		qsci->setMarkerBackgroundColor(readColor(colors, "error-marker", QColor(255, 0, 0, 100)), errMarkerNumber);
+		qsci->setMarkerBackgroundColor(readColor(colors, "bookmark-marker", QColor(150, 200, 255, 100)), bmMarkerNumber); // light blue
         qsci->setIndicatorForegroundColor(readColor(colors, "error-indicator", QColor(255, 0, 0, 100)), errorIndicatorNumber);//red
         qsci->setIndicatorOutlineColor(readColor(colors, "error-indicator-outline", QColor(255, 0, 0, 100)), errorIndicatorNumber);//red
         qsci->setIndicatorForegroundColor(readColor(colors, "find-indicator", QColor(255, 255, 0, 100)), findIndicatorNumber);//yellow
         qsci->setIndicatorOutlineColor(readColor(colors, "find-indicator-outline", QColor(255, 255, 0, 100)), findIndicatorNumber);//yellow
+        qsci->setIndicatorForegroundColor(readColor(colors, "hyperlink-indicator", QColor(139, 24, 168, 100)), hyperlinkIndicatorNumber);//violet
+        qsci->setIndicatorOutlineColor(readColor(colors, "hyperlink-indicator-outline", QColor(139, 24, 168, 100)), hyperlinkIndicatorNumber);//violet
+        qsci->setIndicatorHoverForegroundColor(readColor(colors, "hyperlink-indicator-hover", QColor(139, 24, 168, 100)), hyperlinkIndicatorNumber);//violet
 		qsci->setWhitespaceForegroundColor(readColor(colors, "whitespace-foreground", textColor));
 		qsci->setMarginsBackgroundColor(readColor(colors, "margin-background", paperColor));
 		qsci->setMarginsForegroundColor(readColor(colors, "margin-foreground", textColor));
@@ -352,11 +501,15 @@ void ScintillaEditor::noColor()
 	this->lexer->setColor(Qt::black);
 	qsci->setCaretWidth(2);
 	qsci->setCaretForegroundColor(Qt::black);
-	qsci->setMarkerBackgroundColor(QColor(255, 0, 0, 100), markerNumber);
+	qsci->setMarkerBackgroundColor(QColor(255, 0, 0, 100), errMarkerNumber);
+	qsci->setMarkerBackgroundColor(QColor(150, 200, 255, 100), bmMarkerNumber); // light blue
     qsci->setIndicatorForegroundColor(QColor(255, 0, 0, 128), errorIndicatorNumber);//red
     qsci->setIndicatorOutlineColor(QColor(0, 0, 0, 255), errorIndicatorNumber); // only alpha part is used
     qsci->setIndicatorForegroundColor(QColor(255, 255, 0, 128), findIndicatorNumber);//yellow
     qsci->setIndicatorOutlineColor(QColor(0, 0, 0, 255), findIndicatorNumber); // only alpha part is used
+    qsci->setIndicatorForegroundColor(QColor(139, 24, 168, 128), hyperlinkIndicatorNumber);//violet
+    qsci->setIndicatorOutlineColor(QColor(0, 0, 0, 255), hyperlinkIndicatorNumber); // only alpha part is used
+    qsci->setIndicatorHoverForegroundColor(QColor(139, 24, 168, 128), hyperlinkIndicatorNumber);//violet
 	qsci->setCaretLineBackgroundColor(Qt::white);
 	qsci->setWhitespaceForegroundColor(Qt::black);
 	qsci->setSelectionForegroundColor(Qt::black);
@@ -486,40 +639,42 @@ void ScintillaEditor::initFont(const QString& fontName, uint size)
 
 void ScintillaEditor::initMargin()
 {
-  qsci->setMarginLineNumbers(1, true);
   connect(qsci, SIGNAL(textChanged()), this, SLOT(onTextChanged()));
 }
 
 void ScintillaEditor::onTextChanged()
 {
-  auto s = Settings::Settings::inst();
-  QFontMetrics fontmetrics(this->currentFont);
-  auto enableLineNumbers = s->get(Settings::Settings::enableLineNumbers).toBool();
+	auto s = Settings::Settings::inst();
+	auto enableLineNumbers = s->get(Settings::Settings::enableLineNumbers).toBool();
 
-  if (!enableLineNumbers) qsci->setMarginWidth(1, 20);
-  else qsci->setMarginWidth(1, QString(trunc(log10(qsci->lines()) + 4), '0'));
+	if (enableLineNumbers) {
+		qsci->setMarginWidth(numberMargin, QString(trunc(log10(qsci->lines()) + 2), '0'));
+	} else {
+		qsci->setMarginWidth(numberMargin, 6);
+	}
+	qsci->setMarginLineNumbers(numberMargin, enableLineNumbers);
 }
 
 int ScintillaEditor::updateFindIndicators(const QString &findText, bool visibility)
 {
 	int findwordcount{0};
 
-    auto txt = qsci->text();
+	qsci->SendScintilla(QsciScintilla::SCI_SETINDICATORCURRENT, findIndicatorNumber);
+	qsci->SendScintilla(qsci->SCI_INDICATORCLEARRANGE, 0, qsci->length());
 
-    qsci->SendScintilla(qsci->SCI_INDICATORCLEARRANGE, 0, txt.length());
-
-    auto pos = txt.indexOf(findText);
-    auto len = findText.length();
-    if (visibility && len>0) {
-        while (pos!=-1){
-            findwordcount++;
-            qsci->SendScintilla(qsci->SCI_SETINDICATORCURRENT, findIndicatorNumber);
-            qsci->SendScintilla(qsci->SCI_INDICATORFILLRANGE, pos, len);
-            pos = txt.indexOf(findText,pos+len);
-        };
-    }
-    //qsci->findFirst(findText, false, false, false, true, true, savelineFrom, saveindexFrom);
-    return findwordcount;
+	const auto txt = qsci->text().toUtf8();
+	const auto findTextUtf8 = findText.toUtf8();
+	auto pos = txt.indexOf(findTextUtf8);
+	auto len = findTextUtf8.length();
+	if (visibility && len > 0) {
+		while (pos != -1) {
+			findwordcount++;
+			qsci->SendScintilla(qsci->SCI_SETINDICATORCURRENT, findIndicatorNumber);
+			qsci->SendScintilla(qsci->SCI_INDICATORFILLRANGE, pos, len);
+			pos = txt.indexOf(findTextUtf8, pos + len);
+		};
+	}
+	return findwordcount;
 }
 
 bool ScintillaEditor::find(const QString &expr, bool findNext, bool findBackwards)
@@ -591,7 +746,7 @@ void ScintillaEditor::indentSelection()
 {
 	int lineFrom, lineTo;
 	getRange(&lineFrom, &lineTo);
-	for (int line = lineFrom; line <= lineTo; line++) {
+	for (int line = lineFrom; line <= lineTo; ++line) {
 		qsci->indent(line);
 	}
 }
@@ -600,7 +755,7 @@ void ScintillaEditor::unindentSelection()
 {
 	int lineFrom, lineTo;
 	getRange(&lineFrom, &lineTo);
-	for (int line = lineFrom; line <= lineTo; line++) {
+	for (int line = lineFrom; line <= lineTo; ++line) {
 		qsci->unindent(line);
 	}
 }
@@ -611,7 +766,7 @@ void ScintillaEditor::commentSelection()
 
 	int lineFrom, lineTo;
 	getRange(&lineFrom, &lineTo);
-	for (int line = lineFrom; line <= lineTo; line++) {
+	for (int line = lineFrom; line <= lineTo; ++line) {
 		qsci->insertAt("//", line, 0);
 	}
 
@@ -626,7 +781,7 @@ void ScintillaEditor::uncommentSelection()
 
 	int lineFrom, lineTo;
 	getRange(&lineFrom, &lineTo);
-	for (int line = lineFrom; line <= lineTo; line++) {
+	for (int line = lineFrom; line <= lineTo; ++line) {
 		QString lineText = qsci->text(line);
 		if (lineText.startsWith("//")) {
 			qsci->setSelection(line, 0, line, 2);
@@ -643,59 +798,272 @@ QString ScintillaEditor::selectedText()
 	return qsci->selectedText();
 }
 
-bool ScintillaEditor::eventFilter(QObject* obj, QEvent *e)
+bool ScintillaEditor::eventFilter(QObject *obj, QEvent *e)
 {
-	static bool wasChanged=false;
-	static bool previewAfterUndo=false;
+	bool enableNumberScrollWheel = Settings::Settings::inst()->get(Settings::Settings::enableNumberScrollWheel).toBool();
 
-	if (obj != qsci) return EditorInterface::eventFilter(obj, e);
-
-	if (e->type()==QEvent::KeyPress || e->type()==QEvent::KeyRelease) {
-		auto *ke = static_cast<QKeyEvent*>(e);
-#ifdef Q_OS_MAC
-                int navigateOnNumberModifiers = Qt::AltModifier | Qt::ShiftModifier | Qt :: KeypadModifier;
-#else
-                int navigateOnNumberModifiers = Qt::AltModifier;
-#endif
-		if (ke->modifiers()  == navigateOnNumberModifiers) {
-			switch (ke->key())
+	if(obj == qsci->viewport() && enableNumberScrollWheel)
+	{
+		if(e->type() == QEvent::Wheel)
+		{
+			auto *wheelEvent = static_cast <QWheelEvent*> (e);
+			PRINTDB("%s - modifier: %s",(e->type()==QEvent::Wheel?"Wheel Event":"")%(wheelEvent->modifiers() & Qt::AltModifier?"Alt":"Other Button"));
+			if(handleWheelEventNavigateNumber(wheelEvent))
 			{
-				case Qt::Key_Left:
-				case Qt::Key_Right:
-					if (e->type()==QEvent::KeyPress) {
-						navigateOnNumber(ke->key());
-					}
-					return true;
-
-				case Qt::Key_Up:
-				case Qt::Key_Down:
-					if (e->type()==QEvent::KeyPress) {
-						if (!wasChanged) qsci->beginUndoAction();
-						if (modifyNumber(ke->key())) {
-							wasChanged=true;
-							previewAfterUndo=true;
-						}
-						if (!wasChanged) qsci->endUndoAction();
-					}
-					return true;
+				qsci->SendScintilla(QsciScintilla::SCI_SETCARETWIDTH, 1);
+				return true;
 			}
 		}
-		if (previewAfterUndo && e->type()==QEvent::KeyPress) {
-			int k=ke->key() | ke->modifiers();
-			if (wasChanged) qsci->endUndoAction();
-			wasChanged=false;
-			auto *cmd=qsci->standardCommands()->boundTo(k);
-			if ( cmd && ( cmd->command()==QsciCommand::Undo || cmd->command()==QsciCommand::Redo ) )
-				QTimer::singleShot(0,this,SIGNAL(previewRequest()));
-			else if ( cmd || !ke->text().isEmpty() ) {
-				// any insert or command (but not undo/redo) cancels the preview after undo
-				previewAfterUndo=false;
+		return false;
+	}
+	else if(obj == qsci)
+	{
+		if (e->type() == QEvent::KeyPress || e->type() == QEvent::KeyRelease)
+		{
+		auto *keyEvent = static_cast<QKeyEvent*> (e);
+
+		PRINTDB("%10s - modifiers: %s %s %s %s %s %s",
+				(e->type() == QEvent::KeyPress ? "KeyPress" : "KeyRelease") %
+				(keyEvent->modifiers() & Qt::ShiftModifier ? "SHIFT" : "shift") %
+				(keyEvent->modifiers() & Qt::ControlModifier ? "CTRL" : "ctrl") %
+				(keyEvent->modifiers() & Qt::AltModifier ? "ALT" : "alt") %
+				(keyEvent->modifiers() & Qt::MetaModifier ? "META" : "meta") %
+				(keyEvent->modifiers() & Qt::KeypadModifier ? "KEYPAD" : "keypad") %
+				(keyEvent->modifiers() & Qt::GroupSwitchModifier ? "GROUP" : "group"));
+
+			if (handleKeyEventNavigateNumber(keyEvent))
+			{
+			return true;
+		}
+			if (handleKeyEventBlockCopy(keyEvent))
+			{
+			return true;
+		}
+			if (handleKeyEventBlockMove(keyEvent))
+			{
+			return true;
+		}
+	}
+	return false;
+}
+	else return EditorInterface::eventFilter(obj, e);
+
+	return false;
+}
+
+bool ScintillaEditor::handleKeyEventBlockMove(QKeyEvent *keyEvent)
+{
+	unsigned int modifiers = Qt::ControlModifier | Qt::GroupSwitchModifier;
+
+	if (keyEvent->type() != QEvent::KeyRelease) {
+		return false;
+	}
+
+	if (keyEvent->modifiers() != modifiers) {
+		return false;
+	}
+
+	if (keyEvent->key() != Qt::Key_Up && keyEvent->key() != Qt::Key_Down) {
+		return false;
+	}
+
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+	int lineFrom, indexFrom, lineTo, indexTo;
+	qsci->getSelection(&lineFrom, &indexFrom, &lineTo, &indexTo);
+	if (lineFrom < 0) {
+		lineTo = lineFrom = line;
+		indexFrom = indexTo = 0;
+	}
+	int selectionLineTo = lineTo;
+	if (lineTo > lineFrom && indexTo == 0) {
+		lineTo--;
+	}
+
+	bool up = keyEvent->key() == Qt::Key_Up;
+	int directionOffset = up ? -1 : 1;
+	int lineToMove = up ? lineFrom - 1 : lineTo + 1;
+	if (lineToMove < 0) {
+		return false;
+	}
+
+	qsci->beginUndoAction();
+	QString textToMove = qsci->text(lineToMove);
+	QString text;
+	for (int idx = lineFrom; idx <= lineTo; ++idx) {
+		text.append(qsci->text(idx));
+	}
+	if (lineToMove >= qsci->lines() - 1) {
+		textToMove.append('\n');
+	}
+	text.insert(up ? text.length() : 0, textToMove);
+	qsci->setSelection(std::min(lineToMove, lineFrom), 0, std::max(lineToMove, lineTo) + 1, 0);
+	qsci->replaceSelectedText(text);
+	qsci->setCursorPosition(line + directionOffset, index);
+	qsci->setSelection(lineFrom + directionOffset, indexFrom, selectionLineTo + directionOffset, indexTo);
+	qsci->endUndoAction();
+	return true;
+}
+
+bool ScintillaEditor::handleKeyEventBlockCopy(QKeyEvent *keyEvent)
+{
+	unsigned int modifiers = Qt::ControlModifier | Qt::ShiftModifier;
+
+	if (keyEvent->type() != QEvent::KeyRelease) {
+		return false;
+	}
+
+	if (keyEvent->modifiers() != modifiers) {
+		return false;
+	}
+
+	if (keyEvent->key() != Qt::Key_Up && keyEvent->key() != Qt::Key_Down) {
+		return false;
+	}
+
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+	int lineFrom, indexFrom, lineTo, indexTo;
+	qsci->getSelection(&lineFrom, &indexFrom, &lineTo, &indexTo);
+	if (lineFrom < 0) {
+		lineTo = lineFrom = line;
+		indexFrom = indexTo = 0;
+	}
+
+	bool up = keyEvent->key() == Qt::Key_Up;
+	int selectionLineTo = 0;
+	if (lineTo > lineFrom && indexTo == 0) {
+		lineTo--;
+		selectionLineTo++;
+	}
+	int cursorLine = up ? line : lineTo + 1;
+	int selectionLineFrom = up ? lineFrom : lineTo + 1;
+	selectionLineTo += up ? lineTo : 2 * lineTo - lineFrom + 1;
+
+	qsci->beginUndoAction();
+	QString text;
+	for (int line = lineFrom; line <= lineTo; ++line) {
+		text += qsci->text(line);
+	}
+	if (lineTo + 1 >= qsci->lines()) {
+		text.insert(0, '\n');
+	}
+	qsci->insertAt(text, lineTo + 1, 0);
+	qsci->setCursorPosition(cursorLine, index);
+	qsci->setSelection(selectionLineFrom, indexFrom, selectionLineTo, indexTo);
+	qsci->endUndoAction();
+	return true;
+}
+
+bool ScintillaEditor::handleKeyEventNavigateNumber(QKeyEvent *keyEvent)
+{
+	static bool wasChanged = false;
+	static bool previewAfterUndo = false;
+
+#ifdef Q_OS_MAC
+	unsigned int navigateOnNumberModifiers = Qt::AltModifier | Qt::ShiftModifier | Qt::KeypadModifier;
+#else
+	unsigned int navigateOnNumberModifiers = Qt::AltModifier;
+#endif
+	if (keyEvent->modifiers() == navigateOnNumberModifiers) {
+		switch (keyEvent->key()) {
+		case Qt::Key_Left:
+		case Qt::Key_Right:
+			if (keyEvent->type() == QEvent::KeyPress) {
+				navigateOnNumber(keyEvent->key());
 			}
+			return true;
+
+		case Qt::Key_Up:
+		case Qt::Key_Down:
+			if (keyEvent->type() == QEvent::KeyPress) {
+				if (!wasChanged) qsci->beginUndoAction();
+				if (modifyNumber(keyEvent->key())) {
+					wasChanged = true;
+					previewAfterUndo = true;
+				}
+				if (!wasChanged) qsci->endUndoAction();
+			}
+			return true;
+		}
+	}
+	if (previewAfterUndo && keyEvent->type() == QEvent::KeyPress) {
+		int k = keyEvent->key() | keyEvent->modifiers();
+		if (wasChanged) qsci->endUndoAction();
+		wasChanged = false;
+		auto *cmd = qsci->standardCommands()->boundTo(k);
+		if (cmd && (cmd->command() == QsciCommand::Undo || cmd->command() == QsciCommand::Redo))
+			QTimer::singleShot(0, this, SIGNAL(previewRequest()));
+		else if (cmd || !keyEvent->text().isEmpty()) {
+			// any insert or command (but not undo/redo) cancels the preview after undo
+			previewAfterUndo = false;
 		}
 	}
 	return false;
 }
 
+bool ScintillaEditor::handleWheelEventNavigateNumber (QWheelEvent *wheelEvent)
+{
+	auto modifierNumberScrollWheel = Settings::Settings::inst()->get(Settings::Settings::modifierNumberScrollWheel).toString();
+	bool modifier;	
+	static bool wasChanged = false;
+	static bool previewAfterUndo = false;
+
+	if(modifierNumberScrollWheel=="Alt")
+	{
+		modifier = wheelEvent->modifiers() & Qt::AltModifier;
+	}
+	else if(modifierNumberScrollWheel=="Left Mouse Button")
+	{
+		modifier = wheelEvent->buttons() & Qt::LeftButton;
+	}
+	else
+	{
+		modifier = (wheelEvent->buttons() & Qt::LeftButton) | (wheelEvent->modifiers() & Qt::AltModifier);	
+	}
+
+	if (modifier)
+	 {
+		if (!wasChanged) qsci->beginUndoAction();
+
+		if (wheelEvent->delta() < 0)
+		{
+			if (modifyNumber(Qt::Key_Down)) 
+			{
+				wasChanged = true;
+				previewAfterUndo = true;
+			}
+		}
+		else
+		{
+			// wheelEvent->delta() > 0
+			if (modifyNumber(Qt::Key_Up))
+			{
+				wasChanged = true;
+				previewAfterUndo = true;
+			}
+		}
+		if (!wasChanged) qsci->endUndoAction();
+
+		return true;
+	}
+
+	if (previewAfterUndo)
+	{
+        int k = wheelEvent->buttons() & Qt::LeftButton;
+		if (wasChanged) qsci->endUndoAction();
+		wasChanged = false;
+		auto *cmd = qsci->standardCommands()->boundTo(k);
+		if (cmd && (cmd->command() == QsciCommand::Undo || cmd->command() == QsciCommand::Redo))
+			QTimer::singleShot(0, this, SIGNAL(previewRequest()));
+		else if (cmd || wheelEvent->delta())
+		{
+			// any insert or command (but not undo/redo) cancels the preview after undo
+			previewAfterUndo = false;
+		}
+	}
+	return false;
+}
 
 void ScintillaEditor::navigateOnNumber(int key)
 {
@@ -738,25 +1106,32 @@ bool ScintillaEditor::modifyNumber(int key)
 	int line, index;
 	qsci->getCursorPosition(&line, &index);
 	auto text=qsci->text(line);
-
 	int lineFrom, indexFrom, lineTo, indexTo;
 	qsci->getSelection(&lineFrom, &indexFrom, &lineTo, &indexTo);
 	auto hadSelection=qsci->hasSelectedText();
+	qsci->SendScintilla(QsciScintilla::SCI_SETEMPTYSELECTION);
+	qsci->setCursorPosition(line, index);
 
 	auto begin=QRegExp("[-+]?\\d*\\.?\\d*$").indexIn(text.left(index));
+
+	QRegExp rx("[_a-zA-Z]");
+	auto check = text.mid(begin-1,1);
+	if(rx.exactMatch(check)) return false;
+
 	auto end=text.indexOf(QRegExp("[^0-9.]"),index);
 	if (end<0) end=text.length();
 	auto nr=text.mid(begin,end-begin);
-	if ( !(nr.contains(QRegExp("^[-+]?\\d*\\.?\\d*$")) && nr.contains(QRegExp("\\d"))) ) return false;
+	if ( !(nr.contains(QRegExp("^[-+]?\\d*\\.?\\d+$")) && nr.contains(QRegExp("\\d"))) ) return false;
 	auto sign=nr[0]=='+'||nr[0]=='-';
 	if (nr.endsWith('.')) nr=nr.left(nr.length()-1);
 	auto curpos=index-begin;
+	if(curpos==0 || (curpos==1 && (nr[0]=='+' || nr[0]=='-'))) return false;
 	auto dotpos=nr.indexOf('.');
 	auto decimals=dotpos<0?0:nr.length()-dotpos-1;
 	auto number=(dotpos<0)?nr.toLongLong():(nr.left(dotpos)+nr.mid(dotpos+1)).toLongLong();
 	auto tail=nr.length()-curpos;
 	auto exponent=tail-((dotpos>=curpos)?1:0);
-	long long int step=1;
+	long long int step=Preferences::inst()->getValue("editor/stepSize").toInt();
 	for (int i=exponent; i>0; i--) step*=10;
 
 	switch (key) {
@@ -787,3 +1162,182 @@ bool ScintillaEditor::modifyNumber(int key)
 	emit previewRequest();
 	return true;
 }
+
+void ScintillaEditor::onUserListSelected(const int, const QString &text)
+{
+	if (!templateMap.contains(text)) {
+		return;
+	}
+
+	QString tabReplace = "";
+	if(Settings::Settings::inst()->get(Settings::Settings::indentStyle).toString() == "Spaces")
+	{
+		auto spCount = Settings::Settings::inst()->get(Settings::Settings::indentationWidth).toDouble();
+		tabReplace = QString(spCount, ' ');
+	}
+
+	ScadTemplate &t = templateMap[text];
+	QString content = t.get_text();
+	int cursor_offset = t.get_cursor_offset();
+
+	if(cursor_offset < 0)
+	{
+		if(tabReplace.size() != 0)
+			content.replace("\t", tabReplace);
+
+		cursor_offset = content.indexOf(ScintillaEditor::cursorPlaceHolder);
+		content.remove(cursorPlaceHolder);
+
+		if(cursor_offset == -1)
+			cursor_offset = content.size();
+	}
+	else
+	{
+		if(tabReplace.size() != 0)
+		{
+			int tbCount = content.left(cursor_offset).count("\t");
+			cursor_offset += tbCount * (tabReplace.size() - 1);
+			content.replace("\t", tabReplace);
+		}
+	}
+
+	qsci->insert(content);
+
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+	int pos = qsci->positionFromLineIndex(line, index);
+
+	pos += cursor_offset;
+	int indent_line = line;
+	int indent_width = index;
+	qsci->lineIndexFromPosition(pos, &line, &index);
+	qsci->setCursorPosition(line, index);
+
+	int lines = t.get_text().count("\n");
+	QString indent_char = " ";
+	if(Settings::Settings::inst()->get(Settings::Settings::indentStyle).toString() == "Tabs")
+		indent_char = "\t";
+
+	for (int a = 0; a < lines; ++a) {
+		qsci->insertAt(indent_char.repeated(indent_width), indent_line + a + 1, 0);
+	}
+}
+
+void ScintillaEditor::onAutocompleteChanged(bool state)
+{
+	if(state)
+	{
+		qsci->setAutoCompletionSource(QsciScintilla::AcsAPIs);
+		qsci->setAutoCompletionFillupsEnabled(true);
+		qsci->setCallTipsVisible(10);
+		qsci->setCallTipsStyle(QsciScintilla::CallTipsContext);
+	}
+	else
+	{
+		qsci->setAutoCompletionSource(QsciScintilla::AcsNone);
+		qsci->setAutoCompletionFillupsEnabled(false);
+		qsci->setCallTipsStyle(QsciScintilla::CallTipsNone);
+	}
+}
+
+void ScintillaEditor::onCharacterThresholdChanged(int val)
+{
+	qsci->setAutoCompletionThreshold(val <= 0 ? 1 : val);
+}
+
+void ScintillaEditor::setIndicator(const std::vector<IndicatorData>& indicatorData)
+{
+	qsci->SendScintilla(QsciScintilla::SCI_SETINDICATORCURRENT, hyperlinkIndicatorNumber);
+	qsci->SendScintilla(QsciScintilla::SCI_INDICATORCLEARRANGE, 0, qsci->text().length());
+	this->indicatorData = indicatorData;
+
+	int idx = 0;
+	for (const auto& data : indicatorData) {
+		int pos = qsci->positionFromLineIndex(data.linenr - 1, data.colnr - 1);
+		qsci->SendScintilla(QsciScintilla::SCI_SETINDICATORVALUE, idx + hyperlinkIndicatorOffset);
+		qsci->SendScintilla(QsciScintilla::SCI_INDICATORFILLRANGE, pos, data.nrofchar);
+		idx++;
+	}
+}
+
+void ScintillaEditor::onIndicatorClicked(int line, int col, Qt::KeyboardModifiers state)
+{
+	if (!(state == Qt::ControlModifier || state == (Qt::ControlModifier|Qt::AltModifier)))
+		return;
+
+	qsci->SendScintilla(QsciScintilla::SCI_SETINDICATORCURRENT, hyperlinkIndicatorNumber);
+
+	int pos = qsci->positionFromLineIndex(line, col);
+	int val = qsci->SendScintilla(QsciScintilla::SCI_INDICATORVALUEAT, ScintillaEditor::hyperlinkIndicatorNumber, pos);
+
+	// checking if indicator clicked is hyperlinkIndicator
+	if(val >= hyperlinkIndicatorOffset && val <= hyperlinkIndicatorOffset+indicatorData.size())	{
+		emit hyperlinkIndicatorClicked(val - hyperlinkIndicatorOffset);
+	}
+}
+
+void ScintillaEditor::setCursorPosition(int line, int col)
+{
+	qsci->setCursorPosition(line, col);
+}
+
+void ScintillaEditor::updateSymbolMarginVisibility()
+{
+	if (qsci->markerFindNext(0, 1 << bmMarkerNumber | 1 << errMarkerNumber) < 0) {
+		qsci->setMarginWidth(symbolMargin, 0);
+	} else {
+		qsci->setMarginWidth(symbolMargin, "00");
+	}
+}
+
+void ScintillaEditor::toggleBookmark()
+{
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+
+	unsigned int state = qsci->markersAtLine(line);
+
+	if ((state & (1<<bmMarkerNumber))==0)
+		qsci->markerAdd(line, bmMarkerNumber);
+	else
+		qsci->markerDelete(line, bmMarkerNumber);
+
+	updateSymbolMarginVisibility();
+}
+
+void ScintillaEditor::findMarker(int findStartOffset, int wrapStart, std::function<int(int)> findMarkerFunc)
+{
+	int line, index;
+	qsci->getCursorPosition(&line, &index);
+	line = findMarkerFunc(line + findStartOffset);
+	if (line == -1) {
+		line = findMarkerFunc(wrapStart); // wrap around
+	}
+	if (line != -1) {
+		// make sure we don't wrap into new line
+		int len = qsci->text(line).remove(QRegExp("[\n\r]$")).length();
+		int col = std::min(index, len);
+		qsci->setCursorPosition(line, col);
+	}
+}
+
+void ScintillaEditor::nextBookmark()
+{
+	findMarker(1, 0, [this](int line){ return qsci->markerFindNext(line, 1 << bmMarkerNumber); });
+}
+
+void ScintillaEditor::prevBookmark()
+{
+	findMarker(-1, qsci->lines() - 1, [this](int line){ return qsci->markerFindPrevious(line, 1 << bmMarkerNumber); });
+}
+
+void ScintillaEditor::jumpToNextError()
+{
+	findMarker(1, 0, [this](int line){ return qsci->markerFindNext(line, 1 << errMarkerNumber); });
+}
+
+void ScintillaEditor::setFocus()
+{
+	qsci->setFocus();
+	qsci->SendScintilla(QsciScintilla::SCI_SETFOCUS, true);
+}
